@@ -235,7 +235,7 @@ static void rtJack_AllocateBuffers(RtJackGlobals *p)
                                    * m);
   nBytes = ofs1 + (nBytesPerBuf * (size_t) p->nBuffers);
   /* allocate memory */
-  ptr = (RtJackBuffer**) malloc(nBytes);
+  ptr = (RtJackBuffer**) csound->Malloc(csound, nBytes);
   if (UNLIKELY(ptr == NULL))
     rtJack_Error(csound, CSOUND_MEMORY, Str("memory allocation failure"));
   p->bufs = (RtJackBuffer**) ptr;
@@ -284,14 +284,14 @@ static void rtJack_AllocateBuffers(RtJackGlobals *p)
 static void listPorts(CSOUND *csound, int isOutput){
           int i,n = listDevices(csound,NULL,isOutput);
           CS_AUDIODEVICE *devs = (CS_AUDIODEVICE *)
-            malloc(n*sizeof(CS_AUDIODEVICE));
+            csound->Malloc(csound, n*sizeof(CS_AUDIODEVICE));
           listDevices(csound,devs,isOutput);
           csound->Message(csound, "Jack %s ports:\n",
                           isOutput ? "output" : "input");
           for(i=0; i < n; i++)
             csound->Message(csound, " %d: %s (dac:%s)\n",
                             i, devs[i].device_id, devs[i].device_name);
-          free(devs);
+          csound->Free(csound,devs);
 }
 
 /* register JACK ports */
@@ -307,7 +307,7 @@ static void rtJack_RegisterPorts(RtJackGlobals *p)
     flags = (unsigned long) JackPortIsTerminal;
   if (p->inputEnabled) {
     /* register input ports */
-    for (i = 0; i < p->nChannels; i++) {
+    for (i = 0; i < p->nChannels_i; i++) {
       snprintf(portName, MAX_NAME_LEN + 4, "%s%d", p->inputPortName, i + 1);
       p->inPorts[i] = jack_port_register(p->client, &(portName[0]),
                                          JACK_DEFAULT_AUDIO_TYPE,
@@ -350,6 +350,10 @@ static void openJackStreams(RtJackGlobals *p)
   /* check consistency of parameters */
   if (UNLIKELY(p->nChannels < 1 || p->nChannels > 255))
     rtJack_Error(csound, -1, Str("invalid number of channels"));
+  if (p->inputEnabled) {
+    if (UNLIKELY(p->nChannels_i < 1 || p->nChannels > 255))
+      rtJack_Error(csound, -1, Str("invalid number of input channels"));
+  }
   if (UNLIKELY(p->sampleRate < 1000 || p->sampleRate > 768000))
     rtJack_Error(csound, -1, Str("invalid sample rate"));
   if (UNLIKELY(p->sampleRate != (int) jack_get_sample_rate(p->client))) {
@@ -383,12 +387,14 @@ static void openJackStreams(RtJackGlobals *p)
   for (i = 0; i < p->nBuffers; i++) {
     rtJack_TryLock(p->csound, &(p->bufs[i]->csndLock));
     rtJack_Unlock(p->csound, &(p->bufs[i]->jackLock));
-    for (j = 0; j < p->nChannels; j++) {
-      if (p->inputEnabled) {
+   if (p->inputEnabled) {
+    for (j = 0; j < p->nChannels_i; j++) {
         for (k = 0; k < p->bufSize; k++)
           p->bufs[i]->inBufs[j][k] = (jack_default_audio_sample_t) 0;
       }
-      if (p->outputEnabled) {
+   }
+   if (p->outputEnabled) {
+    for (j = 0; j < p->nChannels; j++) {
         for (k = 0; k < p->bufSize; k++)
           p->bufs[i]->outBufs[j][k] = (jack_default_audio_sample_t) 0;
       }
@@ -436,7 +442,7 @@ static void openJackStreams(RtJackGlobals *p)
                                                  JACK_DEFAULT_AUDIO_TYPE,
                                                  portFlags);
 
-      for (i = 0; i < p->nChannels; i++) {
+      for (i = 0; i < p->nChannels_i; i++) {
         if(portNames[num+i] != NULL){
           csound->Message(csound, Str("connecting channel %d to %s \n"),
                           i,portNames[num+i]);
@@ -505,7 +511,7 @@ static void openJackStreams(RtJackGlobals *p)
       jack_free(portNames);
     }
     else {
-      if(strcmp(p->outDevName, "null") && p->outDevName != NULL){
+      if(p->outDevName != NULL && strcmp(p->outDevName, "null")){
         char dev[128], *dev_final, *sp;
         strncpy(dev, p->outDevName, 128); dev[127]='\0';
         dev_final = dev;
@@ -558,7 +564,7 @@ static void rtJack_CopyDevParams(RtJackGlobals *p,
     if (parm->devName != NULL && parm->devName[0] != (char) 0) {
       /* NOTE: this assumes max. 999 channels (the current limit is 255) */
       nBytes = strlen(parm->devName) + 4;
-      s = (char*) malloc(nBytes+1);
+      s = (char*) csound->Malloc(csound, nBytes+1);
       if (UNLIKELY(s == NULL))
         rtJack_Error(csound, CSOUND_MEMORY, Str("memory allocation failure"));
       strcpy(s, parm->devName);
@@ -573,7 +579,7 @@ static void rtJack_CopyDevParams(RtJackGlobals *p,
     }
     if (isOutput && p->inputEnabled) {
       /* full duplex audio I/O: check consistency of parameters */
-      if (UNLIKELY(p->nChannels != parm->nChannels ||
+      if (UNLIKELY(/*p->nChannels != parm->nChannels ||*/
                    (unsigned int)p->bufSize != parm->bufSamp_SW))
         rtJack_Error(csound, -1,
                      Str("input and output parameters are not consistent"));
@@ -586,7 +592,9 @@ static void rtJack_CopyDevParams(RtJackGlobals *p,
   p->sampleRate = (int) parm->sampleRate;
   if (UNLIKELY((float) p->sampleRate != parm->sampleRate))
     rtJack_Error(csound, -1, Str("sample rate must be an integer"));
-  p->nChannels = parm->nChannels;
+  if(isOutput) p->nChannels = parm->nChannels;
+  else p->nChannels_i = parm->nChannels;
+
   p->bufSize = parm->bufSamp_SW;
   p->nBuffers = (parm->bufSamp_HW + parm->bufSamp_SW - 1) / parm->bufSamp_SW;
 
@@ -606,12 +614,13 @@ static int recopen_(CSOUND *csound, const csRtAudioParams *parm)
   p->inputEnabled = 1;
   /* allocate pointers to input ports */
   p->inPorts = (jack_port_t**)
-    calloc((size_t) p->nChannels, sizeof(jack_port_t*));
+    csound->Calloc(csound, (size_t) p->nChannels_i* sizeof(jack_port_t*));
   if (UNLIKELY(p->inPorts == NULL))
     rtJack_Error(p->csound, CSOUND_MEMORY, Str("memory allocation failure"));
   /* allocate pointers to input port buffers */
   p->inPortBufs = (jack_default_audio_sample_t**)
-    calloc((size_t) p->nChannels, sizeof(jack_default_audio_sample_t*));
+    csound->Calloc(csound,
+                   (size_t) p->nChannels_i * sizeof(jack_default_audio_sample_t*));
   if (UNLIKELY(p->inPortBufs == NULL))
     rtJack_Error(p->csound, CSOUND_MEMORY, Str("memory allocation failure"));
   return 0;
@@ -632,12 +641,13 @@ static int playopen_(CSOUND *csound, const csRtAudioParams *parm)
   p->outputEnabled = 1;
   /* allocate pointers to output ports */
   p->outPorts = (jack_port_t**)
-    calloc((size_t) p->nChannels, sizeof(jack_port_t*));
+    csound->Calloc(csound, (size_t) p->nChannels* sizeof(jack_port_t*));
   if (UNLIKELY(p->outPorts == NULL))
     rtJack_Error(p->csound, CSOUND_MEMORY, Str("memory allocation failure"));
   /* allocate pointers to output port buffers */
   p->outPortBufs = (jack_default_audio_sample_t**)
-    calloc((size_t) p->nChannels, sizeof(jack_default_audio_sample_t*));
+    csound->Calloc(csound,
+                   (size_t) p->nChannels* sizeof(jack_default_audio_sample_t*));
   if (UNLIKELY(p->outPortBufs == NULL))
     rtJack_Error(p->csound, CSOUND_MEMORY, Str("memory allocation failure"));
   /* activate client to start playback */
@@ -657,7 +667,7 @@ static int processCallback(jack_nframes_t nframes, void *arg)
   p = (RtJackGlobals*) arg;
   /* get pointers to port buffers */
   if (p->inputEnabled) {
-    for (i = 0; i < p->nChannels; i++)
+    for (i = 0; i < p->nChannels_i; i++)
       p->inPortBufs[i] = (jack_default_audio_sample_t*)
         jack_port_get_buffer(p->inPorts[i], nframes);
   }
@@ -687,15 +697,17 @@ static int processCallback(jack_nframes_t nframes, void *arg)
     k = (int) nframes - i;
     l = p->bufSize - p->jackBufPos;
     l = (l < k ? l : k);      /* number of frames to copy */
-    for (j = 0; j < p->nChannels; j++) {
-      if (p->inputEnabled) {
+   if (p->inputEnabled) {
+     for (j = 0; j < p->nChannels_i; j++) {
         jack_default_audio_sample_t   *srcp, *dstp;
         srcp = &(p->inPortBufs[j][i]);
         dstp = &(p->bufs[p->jackBufCnt]->inBufs[j][p->jackBufPos]);
         for (k = 0; k < l; k++)
           dstp[k] = srcp[k];
       }
-      if (p->outputEnabled) {
+    }
+    if (p->outputEnabled) {
+    for (j = 0; j < p->nChannels; j++) {
         jack_default_audio_sample_t   *srcp, *dstp;
         srcp = &(p->bufs[p->jackBufCnt]->outBufs[j][p->jackBufPos]);
         dstp = &(p->outPortBufs[j][i]);
@@ -755,7 +767,7 @@ static int rtrecord_(CSOUND *csound, MYFLT *inbuf_, int bytes_)
     else
       rtJack_Abort(csound, p->jackState);
   }
-  nframes = bytes_ / (p->nChannels * (int) sizeof(MYFLT));
+  nframes = bytes_ / (p->nChannels_i * (int) sizeof(MYFLT));
   bufpos = p->csndBufPos;
   bufcnt = p->csndBufCnt;
   for (i = j = 0; i < nframes; i++) {
@@ -775,7 +787,7 @@ static int rtrecord_(CSOUND *csound, MYFLT *inbuf_, int bytes_)
       }
     }
     /* copy audio data */
-    for (k = 0; k < p->nChannels; k++)
+    for (k = 0; k < p->nChannels_i; k++)
       inbuf_[j++] = (MYFLT) p->bufs[bufcnt]->inBufs[k][i];
     if (++bufpos >= p->bufSize) {
       bufpos = 0;
@@ -864,7 +876,7 @@ static void rtJack_DeleteBuffers(RtJackGlobals *p)
     rtJack_DestroyLock(p->csound, &(bufs[i]->csndLock));
     rtJack_DestroyLock(p->csound, &(bufs[i]->jackLock));
   }
-  free((void*) bufs);
+  p->csound->Free(p->csound,(void*) bufs);
 }
 
 /* close the I/O device entirely  */
@@ -896,7 +908,7 @@ static CS_NOINLINE void rtclose_(CSOUND *csound)
     csound->Sleep((size_t) 50);
     /* unregister and free all ports */
     if (p.inPorts != NULL) {
-      for (i = 0; i < p.nChannels; i++) {
+      for (i = 0; i < p.nChannels_i; i++) {
         if (p.inPorts[i] != NULL && p.jackState != 2)
           jack_port_unregister(p.client, p.inPorts[i]);
       }
@@ -914,18 +926,18 @@ static CS_NOINLINE void rtclose_(CSOUND *csound)
   }
   /* free copy of input and output device name */
   if (p.inDevName != NULL)
-    free(p.inDevName);
+    csound->Free(csound,p.inDevName);
   if (p.outDevName != NULL)
-    free(p.outDevName);
+    csound->Free(csound,p.outDevName);
   /* free ports and port buffer pointers */
   if (p.inPorts != NULL)
-    free(p.inPorts);
+    csound->Free(csound,p.inPorts);
   if (p.inPortBufs != NULL)
-    free(p.inPortBufs);
+    csound->Free(csound,p.inPortBufs);
   if (p.outPorts != NULL)
-    free(p.outPorts);
+    csound->Free(csound,p.outPorts);
   if (p.outPortBufs != NULL)
-    free(p.outPortBufs);
+    csound->Free(csound,p.outPortBufs);
   /* free ring buffers */
   rtJack_DeleteBuffers(&p);
   csound->DestroyGlobalVariable(csound, "_rtjackGlobals");
@@ -1224,10 +1236,12 @@ static int midi_in_read(CSOUND *csound,
 
 static int midi_in_close(CSOUND *csound, void *userData){
   jackMidiDevice *dev = (jackMidiDevice *) userData;
-  jack_port_disconnect(dev->client, dev->port);
-  jack_client_close(dev->client);
-  csound->DestroyCircularBuffer(csound, dev->cb);
-  csound->Free(csound, dev);
+  if(dev != NULL) {
+    jack_port_disconnect(dev->client, dev->port);
+    jack_client_close(dev->client);
+    csound->DestroyCircularBuffer(csound, dev->cb);
+    csound->Free(csound, dev);
+  }
   return OK;
 }
 
@@ -1334,19 +1348,93 @@ static int midi_out_write(CSOUND *csound,
 
 static int midi_out_close(CSOUND *csound, void *userData){
   jackMidiDevice *dev = (jackMidiDevice *) userData;
+  if(dev != NULL) {
   jack_port_disconnect(dev->client, dev->port);
   jack_client_close(dev->client);
   csound->DestroyCircularBuffer(csound, dev->cb);
   csound->Free(csound, dev);
+  }
   return OK;
 }
 
 static int listDevicesM(CSOUND *csound, CS_MIDIDEVICE *list,
                         int isOutput){
-  IGN(csound);
+  char            **portNames = (char**) NULL, port[64];
+  unsigned long   portFlags;
+  int             i, n, cnt=0;
+  jack_client_t *jackClient;
+  RtJackGlobals* p =
+    (RtJackGlobals*) csound->QueryGlobalVariableNoCheck(csound,
+                                                        "_rtjackGlobals");
+  char *drv = (char*) (csound->QueryGlobalVariable(csound, "_RTMIDI"));
+
+  if(p->listclient == NULL)
+    p->listclient = jack_client_open("list", JackNoStartServer, NULL);
+
+  jackClient  = p->listclient;
+
+  if(jackClient == NULL) return 0;
+  portFlags = (isOutput ? (unsigned long) JackPortIsInput
+               : (unsigned long) JackPortIsOutput);
+
+  portNames = (char**) jack_get_ports(jackClient,
+                                      (char*) NULL,
+                                      JACK_DEFAULT_MIDI_TYPE,
+                                      portFlags);
+  if(portNames == NULL) {
+    jack_client_close(jackClient);
+    p->listclient = NULL;
+    return 0;
+  }
+
+  memset(port, '\0', 64);
+  for(i=0; portNames[i] != NULL; i++, cnt++) {
+    n = (int) strlen(portNames[i]);
+    strncpy(port, portNames[i], n);
+    port[n] = '\0';
+    if (list != NULL) {
+      strncpy(list[cnt].device_name, port, 63);
+      snprintf(list[cnt].device_id, 63, "%d", cnt);
+      list[cnt].isOutput = isOutput;
+      strcpy(list[i].interface_name, "");
+      strncpy(list[i].midi_module, drv, 63);
+    }
+  }
+  jack_free(portNames);
+  jack_client_close(jackClient);
+  p->listclient = NULL;
+  return cnt;
   return 0;
 }
 
+/*
+static int listDevices(CSOUND *csound, CS_MIDIDEVICE *list, int isOutput){
+  int i, cnt;
+  PmDeviceInfo  *info;
+  char tmp[64];
+  char *drv = (char*) (csound->QueryGlobalVariable(csound, "_RTMIDI"));
+
+  if (UNLIKELY(start_portmidi(csound) != 0))
+      return 0;
+
+  cnt = portMidi_getDeviceCount(isOutput);
+  if (list == NULL) return cnt;
+  for (i = 0; i < cnt; i++) {
+      info = portMidi_getDeviceInfo(i, isOutput);
+      if(info->name != NULL)
+      strncpy(list[i].device_name, info->name, 63);
+      snprintf(tmp, 64, "%d", i);
+      strncpy(list[i].device_id, tmp, 63);
+      list[i].isOutput = isOutput;
+      if (info->interf != NULL)
+         strncpy(list[i].interface_name, info->interf, 63);
+      else strcpy(list[i].interface_name, "");
+     strncpy(list[i].midi_module, drv, 63);
+  }
+  return cnt;
+}
+*/
+
 
 PUBLIC int csoundModuleDestroy(CSOUND *csound)
 {
