@@ -108,6 +108,7 @@ static void log(CSOUND *csound, const char *format,...)
     va_list args;
     va_start(args, format);
     if(csound) {
+      if (csound->GetMessageLevel(csound) & WARNMSG)
         csound->MessageV(csound, 0, format, args);
     } else {
         vfprintf(stdout, format, args);
@@ -142,7 +143,8 @@ static MYFLT profile_original(MYFLT fi, MYFLT bwi)
     return exp(-x)/bwi;
 };
 
-static MYFLT profile(int shape, MYFLT fi, MYFLT bwi,  MYFLT a)
+// profile(p9_profile_shape, profile_sample_index_normalized, bandwidth_samples, p10_profile_parameter);
+static MYFLT profile(int shape, MYFLT fi, MYFLT bwi, MYFLT a)
 {
     MYFLT x = fi / bwi;
     MYFLT y = 0;
@@ -151,11 +153,17 @@ static MYFLT profile(int shape, MYFLT fi, MYFLT bwi,  MYFLT a)
         y = std::exp(-(x * x * a));
         break;
     case 2:
+        // The idea is to take profile 1 and simply say y goes to 0 if below a and to 1 if above a.
         y = std::exp(-(x * x * a));
-        if(y < a) {
-            y = 0.0;
+        if(a < 0.00001) {
+            a = 0.00001;
+        } else if (a > 0.99999) {
+            a = 0.99999;
+        }
+        if (y < a) {
+            y = 0;
         } else {
-            y = 1.0;
+            y = 1;
         }
         break;
     case 3:
@@ -384,7 +392,7 @@ static base_function_t get_base_function(int index)
 
     index--;
     base_function_t functions[] = {
-        base_function_triangle,
+        base_function_triangle,comment
         base_function_pulse,
         base_function_saw,
         base_function_power,
@@ -458,7 +466,7 @@ extern "C" {
         MYFLT p8_harmonic_stretch = ff->e.p[8];
         int p9_profile_shape = (int) ff->e.p[9];
         //base_function_t base_function = get_base_function(p9_profile_shape);
-        int p10_profile_parameter = (int) ff->e.p[10];
+        MYFLT p10_profile_parameter = ff->e.p[10];
         MYFLT samplerate = csound->GetSr(csound);
         log(csound, "samplerate:                  %12d\n", (int) samplerate);
         log(csound, "p1_function_table_number:            %9.4f\n",
@@ -503,7 +511,7 @@ extern "C" {
             warn(csound, "  frequency_sample_index_normalized: %9.4f\n", frequency_sample_index_normalized);
             warn(csound, "  partial_frequency_index:   %12d\n", partial_frequency_index);
             warn(csound, "  bandwidth_Hz:                      %9.4f\n", bandwidth_Hz);
-            warn(csound, "  bandwidth_samples:                 %9.4f\n", bandwidth_samples);
+            warn(csound, "  bandwidth_samples:                  %12.8f\n", bandwidth_samples);
             for (int fft_sample_index = 0; fft_sample_index < complexN; ++fft_sample_index) {
                 MYFLT fft_sample_index_normalized = ((MYFLT) fft_sample_index) / ((MYFLT) N);
                 MYFLT profile_sample_index_normalized = fft_sample_index_normalized - frequency_sample_index_normalized;
@@ -542,6 +550,9 @@ extern "C" {
         { NULL, NULL }
     };
 
-    FLINKAGE_BUILTIN(padsynth_gens)
+    PUBLIC NGFENS *csound_fgen_init(CSOUND *csound)                         \
+    {
+         return padsynth_gens;
+    }
 
 };
