@@ -23,13 +23,14 @@
 
 /* Collection of physical modelled instruments */
 
-#include "csdl.h"
+//#include "csdl.h"
+#include "csoundCore.h"
 #include "clarinet.h"
 #include "flute.h"
 #include "bowed.h"
 #include "brass.h"
 #include <math.h>
-
+#include "interlocks.h"
 /* ************************************** */
 /*  Waveguide Clarinet model ala Smith    */
 /*  after McIntyre, Schumacher, Woodhouse */
@@ -51,7 +52,7 @@
 /*        more for information.               */
 /**********************************************/
 
-static MYFLT ReedTabl_LookUp(ReedTabl *r, MYFLT deltaP)
+static inline MYFLT ReedTabl_LookUp(ReedTabl *r, MYFLT deltaP)
     /*   Perform "Table Lookup" by direct clipped  */
     /*   linear function calculation               */
 {   /*   deltaP is differential reed pressure      */
@@ -104,31 +105,31 @@ void OneZero_setCoeff(OneZero* z, MYFLT aValue)
       z->sgain = z->gain / (FL(1.0) - z->zeroCoeff);
 }
 
-void OneZero_print(CSOUND *csound, OneZero *p)
-{
-    csound->Message(csound,
-                    "OneZero: gain=%f inputs=%f zeroCoeff=%f sgain=%f\n",
-                    p->gain, p->inputs, p->zeroCoeff, p->sgain);
-}
+/* void OneZero_print(CSOUND *csound, OneZero *p) */
+/* { */
+/*     csound->Message(csound, */
+/*                     "OneZero: gain=%f inputs=%f zeroCoeff=%f sgain=%f\n", */
+/*                     p->gain, p->inputs, p->zeroCoeff, p->sgain); */
+/* } */
 
 /* *********************************************************************** */
 int clarinset(CSOUND *csound, CLARIN *p)
 {
     FUNC        *ftp;
 
-    if ((ftp = csound->FTFind(csound, p->ifn)) != NULL) p->vibr = ftp;
+    if (LIKELY((ftp = csound->FTnp2Find(csound, p->ifn)) != NULL)) p->vibr = ftp;
     else {                                      /* Expect sine wave */
       return csound->InitError(csound, Str("No table for Clarinet"));
     }
     if (*p->lowestFreq>=FL(0.0)) {      /* Skip initialisation */
       if (*p->lowestFreq)
-        p->length = (long) (csound->esr / *p->lowestFreq + FL(1.0));
+        p->length = (int32) (CS_ESR / *p->lowestFreq + FL(1.0));
       else if (*p->frequency)
-        p->length = (long) (csound->esr / *p->frequency + FL(1.0));
+        p->length = (int32) (CS_ESR / *p->frequency + FL(1.0));
       else {
-        csound->Message(csound, Str("No base frequency for clarinet "
+        csound->Warning(csound, Str("No base frequency for clarinet "
                                     "-- assuming 50Hz\n"));
-        p->length = (long) (csound->esr / FL(50.0) + FL(1.0));
+        p->length = (int32) (CS_ESR / FL(50.0) + FL(1.0));
       }
       make_DLineL(csound, &p->delayLine, p->length);
       p->reedTable.offSet = FL(0.7);
@@ -139,15 +140,17 @@ int clarinset(CSOUND *csound, CLARIN *p)
     /*    p->noiseGain = 0.2f; */       /* Arguemnts; suggested values? */
     /*    p->vibrGain = 0.1f; */
       {
-        int relestim = (int)(csound->ekr * FL(0.1));
+        int relestim = (int)(CS_EKR * FL(0.1));
         /* 1/10th second decay extention */
         if (relestim > p->h.insdshead->xtratim)
           p->h.insdshead->xtratim = relestim;
       }
-      p->kloop = (int) ((long) (p->h.insdshead->offtim * csound->ekr)
-                        - (long) (csound->ekr * *p->attack));
+      p->kloop = (int) ((int32) (p->h.insdshead->offtim * CS_EKR)
+                        - (int32) (CS_EKR * *p->attack));
+#ifdef BETA
       csound->Message(csound, "offtim=%f  kloop=%d\n",
                               p->h.insdshead->offtim, p->kloop);
+#endif
       p->envelope.rate = FL(0.0);
       p->v_time = 0;
     }
@@ -157,7 +160,9 @@ int clarinset(CSOUND *csound, CLARIN *p)
 int clarin(CSOUND *csound, CLARIN *p)
 {
     MYFLT *ar = p->ar;
-    int   n,nsmps = csound->ksmps;
+    uint32_t offset = p->h.insdshead->ksmps_offset;
+    uint32_t early  = p->h.insdshead->ksmps_no_end;
+    uint32_t n, nsmps = CS_KSMPS;
     MYFLT amp = (*p->amp)*AMP_RSCALE; /* Normalise */
     MYFLT nGain = *p->noiseGain;
     int v_len = (int)p->vibr->flen;
@@ -166,71 +171,77 @@ int clarin(CSOUND *csound, CLARIN *p)
     MYFLT vTime = p->v_time;
 
     if (p->envelope.rate==FL(0.0)) {
-      p->envelope.rate =  amp /(*p->attack*csound->esr);
+      p->envelope.rate =  amp /(*p->attack*CS_ESR);
       p->envelope.value = p->envelope.target = FL(0.55) + amp*FL(0.30);
     }
     p->outputGain = amp + FL(0.001);
     DLineL_setDelay(&p->delayLine, /* length - approx filter delay */
-        (csound->esr/ *p->frequency) * FL(0.5) - FL(1.5));
+        (CS_ESR/ *p->frequency) * FL(0.5) - FL(1.5));
     p->v_rate = *p->vibFreq * p->vibr->flen * csound->onedsr;
                                 /* Check to see if into decay yet */
     if (p->kloop>0 && p->h.insdshead->relesing) p->kloop=1;
     if ((--p->kloop) == 0) {
       p->envelope.state = 1;  /* Start change */
-      p->envelope.rate = p->envelope.value / (*p->dettack * csound->esr);
+      p->envelope.rate = p->envelope.value / (*p->dettack * CS_ESR);
       p->envelope.target =  FL(0.0);
+#ifdef BETA
       csound->Message(csound, "Set off phase time = %f Breath v,r = %f, %f\n",
-                              (MYFLT) csound->kcounter * csound->onedkr,
+                              (MYFLT) CS_KCNT * CS_ONEDKR,
                               p->envelope.value, p->envelope.rate);
+#endif
     }
+    if (UNLIKELY(offset)) memset(ar, '\0', offset*sizeof(MYFLT));
+    if (UNLIKELY(early)) {
+      nsmps -= early;
+      memset(&ar[nsmps], '\0', early*sizeof(MYFLT));
+    }
+    for (n=offset;n<nsmps;n++) {
+      MYFLT   pressureDiff;
+      MYFLT   breathPressure;
+      int32    temp;
+      MYFLT   temp_time, alpha;
+      MYFLT   nextsamp;
+      MYFLT   v_lastOutput;
+      MYFLT   lastOutput;
 
-    for (n=0;n<nsmps;n++) {
-        MYFLT   pressureDiff;
-        MYFLT   breathPressure;
-        long    temp;
-        MYFLT   temp_time, alpha;
-        MYFLT   nextsamp;
-        MYFLT   v_lastOutput;
-        MYFLT   lastOutput;
+      breathPressure = Envelope_tick(&p->envelope);
+      breathPressure += breathPressure * nGain * Noise_tick(csound,&p->noise);
+                                     /* Tick on vibrato table   */
+      vTime += p->v_rate;            /*  Update current time    */
+      while (vTime >= v_len)         /*  Check for end of sound */
+        vTime -= v_len;              /*  loop back to beginning */
+      while (vTime < FL(0.0))        /*  Check for end of sound */
+        vTime += v_len;              /*  loop back to beginning */
 
-        breathPressure = Envelope_tick(&p->envelope);
-        breathPressure += breathPressure * nGain * Noise_tick(csound,&p->noise);
-                                       /* Tick on vibrato table   */
-        vTime += p->v_rate;            /*  Update current time    */
-        while (vTime >= v_len)         /*  Check for end of sound */
-          vTime -= v_len;              /*  loop back to beginning */
-        while (vTime < FL(0.0))        /*  Check for end of sound */
-          vTime += v_len;              /*  loop back to beginning */
-
-        temp_time = vTime;
+      temp_time = vTime;
 
 #ifdef have_phase
-        if (p->v_phaseOffset != FL(0.0)) {
-          temp_time += p->v_phaseOffset;   /*  Add phase offset       */
-          while (temp_time >= v_len)       /*  Check for end of sound */
-            temp_time -= v_len;            /*  loop back to beginning */
-          while (temp_time < FL(0.0))      /*  Check for end of sound */
-            temp_time += v_len;            /*  loop back to beginning */
-        }
+      if (p->v_phaseOffset != FL(0.0)) {
+        temp_time += p->v_phaseOffset;   /*  Add phase offset       */
+        while (temp_time >= v_len)       /*  Check for end of sound */
+          temp_time -= v_len;            /*  loop back to beginning */
+        while (temp_time < FL(0.0))      /*  Check for end of sound */
+          temp_time += v_len;            /*  loop back to beginning */
+      }
 #endif
-        temp = (long) temp_time;    /*  Integer part of time address    */
-                                    /*  fractional part of time address */
-        alpha = temp_time - (MYFLT)temp;
-        v_lastOutput = v_data[temp]; /* Do linear interpolation */
-                        /*  same as alpha*data[temp+1] + (1-alpha)data[temp] */
-        v_lastOutput += (alpha * (v_data[temp+1] - v_lastOutput));
-                                /* End of vibrato tick */
-        breathPressure += breathPressure * vibGain * v_lastOutput;
-        pressureDiff = OneZero_tick(&p->filter, /* differential pressure  */
-                                   DLineL_lastOut(&p->delayLine));
-        pressureDiff = (-FL(0.95)*pressureDiff) - breathPressure;
-                                    /* of reflected and mouth */
-        nextsamp = pressureDiff * ReedTabl_LookUp(&p->reedTable,pressureDiff);
-        nextsamp =  breathPressure + nextsamp;
-                                  /* perform scattering in economical way */
-        lastOutput = DLineL_tick(&p->delayLine, nextsamp);
-        lastOutput *= p->outputGain;
-        ar[n] = lastOutput*AMP_SCALE;
+      temp = (int32) temp_time;    /*  Integer part of time address    */
+                                   /*  fractional part of time address */
+      alpha = temp_time - (MYFLT)temp;
+      v_lastOutput = v_data[temp]; /* Do linear interpolation */
+      /*  same as alpha*data[temp+1] + (1-alpha)data[temp] */
+      v_lastOutput += (alpha * (v_data[temp+1] - v_lastOutput));
+      /* End of vibrato tick */
+      breathPressure += breathPressure * vibGain * v_lastOutput;
+      pressureDiff = OneZero_tick(&p->filter, /* differential pressure  */
+                                  DLineL_lastOut(&p->delayLine));
+      pressureDiff = (-FL(0.95)*pressureDiff) - breathPressure;
+      /* of reflected and mouth */
+      nextsamp = pressureDiff * ReedTabl_LookUp(&p->reedTable,pressureDiff);
+      nextsamp =  breathPressure + nextsamp;
+      /* perform scattering in economical way */
+      lastOutput = DLineL_tick(&p->delayLine, nextsamp);
+      lastOutput *= p->outputGain;
+      ar[n] = lastOutput*AMP_SCALE;
     }
     p->v_time = vTime;
 
@@ -260,7 +271,7 @@ int clarin(CSOUND *csound, CLARIN *p)
 /* nomial calculation.                        */
 /**********************************************/
 
-static MYFLT JetTabl_lookup(MYFLT sample) /* Perform "Table Lookup"  */
+static inline MYFLT JetTabl_lookup(MYFLT sample) /* Perform "Table Lookup"  */
 {                                  /* By Polynomial Calculation */
                                    /* (x^3 - x) approximates sigmoid of jet */
     MYFLT j = sample * (sample*sample - FL(1.0));
@@ -272,21 +283,26 @@ static MYFLT JetTabl_lookup(MYFLT sample) /* Perform "Table Lookup"  */
 int fluteset(CSOUND *csound, FLUTE *p)
 {
     FUNC        *ftp;
-    long        length;
+    int32        length;
 
-    if ((ftp = csound->FTFind(csound, p->ifn)) != NULL) p->vibr = ftp;
+    if (LIKELY((ftp = csound->FTnp2Find(csound, p->ifn)) != NULL)) p->vibr = ftp;
     else {                                   /* Expect sine wave */
       return csound->InitError(csound, Str("No table for Flute"));
     }
     if (*p->lowestFreq>=FL(0.0)) {      /* Skip initialisation?? */
-      if (*p->lowestFreq!=FL(0.0))
-        length = (long) (csound->esr / *p->lowestFreq + FL(1.0));
-      else if (*p->frequency!=FL(0.0))
-        length = (long) (csound->esr / *p->frequency + FL(1.0));
+      if (*p->lowestFreq!=FL(0.0)) {
+        length = (int32) (CS_ESR / *p->lowestFreq + FL(1.0));
+        p->limit = *p->lowestFreq;
+      }
+      else if (*p->frequency!=FL(0.0)) {
+        length = (int32) (CS_ESR / *p->frequency + FL(1.0));
+        p->limit = *p->frequency;
+      }
       else {
-        csound->Message(csound, Str("No base frequency for flute "
+        csound->Warning(csound, Str("No base frequency for flute "
                                     "-- assumed to be 50Hz\n"));
-        length = (long) (csound->esr / FL(50.0) + FL(1.0));
+        length = (int32) (CS_ESR / FL(50.0) + FL(1.0));
+        p->limit = FL(50.0);
       }
       make_DLineL(csound, &p->boreDelay, length);
       length = length >> 1;        /* ??? really; yes from later version */
@@ -318,8 +334,8 @@ int fluteset(CSOUND *csound, FLUTE *p)
       p->maxPress = FL(2.3) / FL(0.8);
       p->outputGain = FL(1.001);
       ADSR_keyOn(&p->adsr);
-      p->kloop = (MYFLT)((int)(p->h.insdshead->offtim*csound->ekr -
-                               csound->ekr*(*p->dettack)));
+      p->kloop = (MYFLT)((int)(p->h.insdshead->offtim*CS_EKR -
+                               CS_EKR*(*p->dettack)));
 
       p->lastFreq = FL(0.0);
       p->lastJet = -FL(1.0);
@@ -332,7 +348,9 @@ int fluteset(CSOUND *csound, FLUTE *p)
 int flute(CSOUND *csound, FLUTE *p)
 {
     MYFLT       *ar = p->ar;
-    int         n,nsmps = csound->ksmps;
+    uint32_t offset = p->h.insdshead->ksmps_offset;
+    uint32_t early  = p->h.insdshead->ksmps_no_end;
+    uint32_t n, nsmps = CS_KSMPS;
     MYFLT       amp = (*p->amp)*AMP_RSCALE; /* Normalise */
     MYFLT       temp;
     int         v_len = (int)p->vibr->flen;
@@ -352,32 +370,41 @@ int flute(CSOUND *csound, FLUTE *p)
                                 /* Start SetFreq */
     if (p->lastFreq != *p->frequency) { /* It changed */
       p->lastFreq = *p->frequency;
+      if (p->limit>p->lastFreq) {
+        p->lastFreq = p->limit;
+        csound->Warning(csound, Str("frequency too low, set to minimum"));
+      }
       p->lastJet = *p->jetRatio;
       /* freq = (2/3)*p->frequency as we're overblowing here */
       /* but 1/(2/3) is 1.5 so multiply for speed */
                             /* Length - approx. filter delay */
-      temp = FL(1.5)* csound->esr / p->lastFreq - FL(2.0);
+      temp = FL(1.5)* CS_ESR / p->lastFreq - FL(2.0);
       DLineL_setDelay(&p->boreDelay, temp); /* Length of bore tube */
       DLineL_setDelay(&p->jetDelay, temp * p->lastJet); /* jet delay shorter */
     }
     else if (*p->jetRatio != p->lastJet) { /* Freq same but jet changed */
       p->lastJet = *p->jetRatio;
                                             /* Length - approx. filter delay */
-      temp = FL(1.5)* csound->esr / p->lastFreq - FL(2.0);
+      temp = FL(1.5)* CS_ESR / p->lastFreq - FL(2.0);
       DLineL_setDelay(&p->jetDelay, temp * p->lastJet); /* jet delay shorter */
     }
                                 /* End SetFreq */
 
     if (p->kloop>FL(0.0) && p->h.insdshead->relesing) p->kloop=FL(1.0);
     if ((--p->kloop) == 0) {
-      p->adsr.releaseRate = p->adsr.value / (*p->dettack * csound->esr);
+      p->adsr.releaseRate = p->adsr.value / (*p->dettack * CS_ESR);
       p->adsr.target = FL(0.0);
       p->adsr.rate = p->adsr.releaseRate;
       p->adsr.state = RELEASE;
     }
     noisegain = *p->noiseGain; jetRefl = *p->jetRefl; endRefl = *p->endRefl;
-    for (n=0;n<nsmps;n++) {
-      long      temp;
+    if (UNLIKELY(offset)) memset(ar, '\0', offset*sizeof(MYFLT));
+    if (UNLIKELY(early)) {
+      nsmps -= early;
+      memset(&ar[nsmps], '\0', early*sizeof(MYFLT));
+    }
+    for (n=offset;n<nsmps;n++) {
+      int32     temp;
       MYFLT     temf;
       MYFLT     temp_time, alpha;
       MYFLT     pressDiff;
@@ -407,8 +434,8 @@ int flute(CSOUND *csound, FLUTE *p)
       }
 #endif
 
-      temp = (long) temp_time;        /*  Integer part of time address    */
-                                      /*  fractional part of time address */
+      temp = (int32) temp_time;        /*  Integer part of time address    */
+                                       /*  fractional part of time address */
       alpha = temp_time - (MYFLT)temp;
       v_lastOutput = v_data[temp];    /* Do linear interpolation */
       /*  same as alpha*data[temp+1] + (1-alpha)data[temp] */
@@ -459,7 +486,7 @@ MYFLT BowTabl_lookup(CSOUND *csound, BowTabl *b, MYFLT sample)
     MYFLT input;
     input = sample /* + b->offSet*/ ;          /*  add bias to sample      */
     input *= b->slope;                         /*  scale it                */
-    lastOutput = (MYFLT)fabs(input) + FL(0.75); /*  below min delta, frict = 1 */
+    lastOutput = FABS(input) + FL(0.75); /*  below min delta, frict = 1 */
     lastOutput = csound->intpow(lastOutput,-4L);
 /* if (lastOutput < FL(0.0) ) lastOutput = FL(0.0); */ /* minimum frict is 0.0 */
     if (lastOutput > FL(1.0)) lastOutput = FL(1.0); /*  maximum friction is 1.0 */
@@ -468,23 +495,28 @@ MYFLT BowTabl_lookup(CSOUND *csound, BowTabl *b, MYFLT sample)
 
 int bowedset(CSOUND *csound, BOWED *p)
 {
-    long        length;
+    int32        length;
     FUNC        *ftp;
     MYFLT       amp = (*p->amp)*AMP_RSCALE; /* Normalise */
 
-    if ((ftp = csound->FTFind(csound, p->ifn)) != NULL) p->vibr = ftp;
+    if (LIKELY((ftp = csound->FTnp2Find(csound, p->ifn)) != NULL)) p->vibr = ftp;
     else {                                      /* Expect sine wave */
       return csound->InitError(csound, Str("No table for wgbow vibrato"));
     }
     if (*p->lowestFreq>=FL(0.0)) {      /* If no init skip */
-      if (*p->lowestFreq!=FL(0.0))
-        length = (long) (csound->esr / *p->lowestFreq + FL(1.0));
-      else if (*p->frequency!=FL(0.0))
-        length = (long) (csound->esr / *p->frequency + FL(1.0));
+      if (*p->lowestFreq!=FL(0.0)) {
+        length = (int32) (CS_ESR / *p->lowestFreq + FL(1.0));
+        p->limit = *p->lowestFreq;
+      }
+      else if (*p->frequency!=FL(0.0)) {
+        length = (int32) (CS_ESR / *p->frequency + FL(1.0));
+        p->limit = *p->frequency;
+      }
       else {
-        csound->Message(csound, Str("unknown lowest frequency for bowed string "
+        csound->Warning(csound, Str("unknown lowest frequency for bowed string "
                                     "-- assuming 50Hz\n"));
-        length = (long) (csound->esr / FL(50.0) + FL(1.0));
+        length = (int32) (CS_ESR / FL(50.0) + FL(1.0));
+        p->limit = FL(50.0);
       }
       make_DLineL(csound, &p->neckDelay, length);
       length = length >> 1; /* Unsure about this; seems correct in later code */
@@ -529,7 +561,9 @@ int bowedset(CSOUND *csound, BOWED *p)
 int bowed(CSOUND *csound, BOWED *p)
 {
     MYFLT       *ar = p->ar;
-    int         n,nsmps = csound->ksmps;
+    uint32_t offset = p->h.insdshead->ksmps_offset;
+    uint32_t early  = p->h.insdshead->ksmps_no_end;
+    uint32_t n, nsmps = CS_KSMPS;
     MYFLT       amp = (*p->amp)*AMP_RSCALE; /* Normalise */
     MYFLT       maxVel;
     int         freq_changed = 0;
@@ -545,8 +579,13 @@ int bowed(CSOUND *csound, BOWED *p)
                                 /* Set Frequency if changed */
     if (p->lastfreq != *p->frequency) {
       /* delay - approx. filter delay */
-      p->lastfreq = *p->frequency;
-      p->baseDelay = csound->esr / p->lastfreq - FL(4.0);
+      if (p->limit<=*p->frequency)
+        p->lastfreq = *p->frequency;
+      else {
+        p->lastfreq = p->limit;
+        csound->Warning(csound, Str("frequency too low, set to minimum"));
+      }
+      p->baseDelay = CS_ESR / p->lastfreq - FL(4.0);
       freq_changed = 1;
     }
     if (p->lastbeta != *p->betaRatio ||
@@ -566,7 +605,12 @@ int bowed(CSOUND *csound, BOWED *p)
       p->adsr.state = RELEASE;
     }
 
-    for (n=0;n<nsmps;n++) {
+    if (UNLIKELY(offset)) memset(ar, '\0', offset*sizeof(MYFLT));
+    if (UNLIKELY(early)) {
+      nsmps -= early;
+      memset(&ar[nsmps], '\0', early*sizeof(MYFLT));
+    }
+    for (n=offset;n<nsmps;n++) {
       MYFLT     bowVelocity;
       MYFLT     bridgeRefl=FL(0.0), nutRefl=FL(0.0);
       MYFLT     newVel=FL(0.0), velDiff=FL(0.0), stringVel=FL(0.0);
@@ -585,7 +629,7 @@ int bowed(CSOUND *csound, BOWED *p)
       DLineL_tick(&p->bridgeDelay, nutRefl + newVel);   /*   propagations  */
 
       if (*p->vibAmt > FL(0.0)) {
-        long    temp;
+        int32    temp;
         MYFLT   temp_time, alpha;
                                 /* Tick on vibrato table */
         p->v_time += p->v_rate;              /*  Update current time    */
@@ -605,7 +649,7 @@ int bowed(CSOUND *csound, BOWED *p)
             temp_time += p->vibr->flen;      /*  loop back to beginning */
         }
 #endif
-        temp = (long) temp_time;    /*  Integer part of time address    */
+        temp = (int32) temp_time;    /*  Integer part of time address    */
                                 /*  fractional part of time address */
         alpha = temp_time - (MYFLT)temp;
         p->v_lastOutput = p->vibr->ftable[temp]; /* Do linear interpolation */
@@ -653,7 +697,7 @@ int bowed(CSOUND *csound, BOWED *p)
 /*  often (each sample)).                                                   */
 /****************************************************************************/
 
-void make_DLineA(CSOUND *csound, DLineA *p, long max_length)
+void make_DLineA(CSOUND *csound, DLineA *p, int32 max_length)
 {
     p->length = max_length;
     csound->AuxAlloc(csound, max_length * sizeof(MYFLT), &p->inputs);
@@ -669,12 +713,10 @@ int DLineA_setDelay(CSOUND *csound, DLineA *p, MYFLT lag)
   /* outPoint chases inpoint + 2 for interp and other        */
     outputPointer = (MYFLT)p->inPoint - lag + FL(2.0);
 
-    if (p->length<=0) {
-      return csound->PerfError(csound, Str("DlineA not initialised"));
-    }
+    if (UNLIKELY(p->length<=0)) goto err1;
     while (outputPointer<0)
         outputPointer += p->length;        /* modulo table length            */
-    p->outPoint = (long) outputPointer;    /* Integer part of delay          */
+    p->outPoint = (int32) outputPointer;    /* Integer part of delay          */
     p->alpha = FL(1.0) + p->outPoint - outputPointer;/* fractional part of delay */
     if (p->alpha<FL(0.1)) {
       outputPointer += FL(1.0);             /*  Hack to avoid pole/zero       */
@@ -683,6 +725,9 @@ int DLineA_setDelay(CSOUND *csound, DLineA *p, MYFLT lag)
     }
     p->coeff = (FL(1.0)-p->alpha)/(FL(1.0)+p->alpha); /* coefficient for all pass*/
     return 0;
+ err1:
+    csound->ErrorMsg(csound, Str("DlineA not initialised"));
+    return NOTOK;
 }
 
 MYFLT DLineA_tick(DLineA *p, MYFLT sample)   /*   Take sample, yield sample */
@@ -747,20 +792,25 @@ int brassset(CSOUND *csound, BRASS *p)
     FUNC        *ftp;
     MYFLT amp = (*p->amp)*AMP_RSCALE; /* Normalise */
 
-    if ((ftp = csound->FTFind(csound, p->ifn)) != NULL) p->vibr = ftp;
+    if (LIKELY((ftp = csound->FTnp2Find(csound, p->ifn)) != NULL)) p->vibr = ftp;
     else {                                      /* Expect sine wave */
       return csound->InitError(csound, Str("No table for Brass"));
     }
     p->frq = *p->frequency;     /* Remember */
     if (*p->lowestFreq>=FL(0.0)) {
-      if (*p->lowestFreq!=FL(0.0))
-        p->length = (long) (csound->esr / *p->lowestFreq + FL(1.0));
-      else if (p->frq!=FL(0.0))
-        p->length = (long) (csound->esr / p->frq + FL(1.0));
+      if (*p->lowestFreq!=FL(0.0)) {
+        p->length = (int32) (CS_ESR / *p->lowestFreq + FL(1.0));
+        p->limit = *p->lowestFreq;
+      }
+      else if (p->frq!=FL(0.0)) {
+        p->length = (int32) (CS_ESR / p->frq + FL(1.0));
+        p->limit = p->frq;
+      }
       else {
-        csound->Message(csound, Str("No base frequency for brass "
+        csound->Warning(csound, Str("No base frequency for brass "
                                     "-- assumed to be 50Hz\n"));
-        p->length = (long) (csound->esr / FL(50.0) + FL(1.0));
+        p->length = (int32) (CS_ESR / FL(50.0) + FL(1.0));
+        p->limit = FL(50.0);
       }
       make_DLineA(csound, &p->delayLine, p->length);
       make_LipFilt(&p->lipFilter);
@@ -775,7 +825,7 @@ int brassset(CSOUND *csound, BRASS *p)
       ADSR_keyOn(&p->adsr);
 
       /* Set frequency */
-      /*      p->slideTarget = (csound->esr / p->frq * FL(2.0)) + 3.0f; */
+      /*      p->slideTarget = (CS_ESR / p->frq * FL(2.0)) + 3.0f; */
       /* fudge correction for filter delays */
       /*      DLineA_setDelay(&p->delayLine, p->slideTarget);*/
       /* we'll play a harmonic  */
@@ -788,13 +838,13 @@ int brassset(CSOUND *csound, BRASS *p)
       /*                     p->lipTarget * (MYFLT)pow(4.0,
                                                        (2.0* p->lipT) -1.0)); */
       {
-        int relestim = (int)(csound->ekr * FL(0.1));
+        int relestim = (int)(CS_EKR * FL(0.1));
         /* 1/10th second decay extention */
         if (relestim > p->h.insdshead->xtratim)
           p->h.insdshead->xtratim = relestim;
       }
-      p->kloop = (int) ((long) (p->h.insdshead->offtim * csound->ekr)
-                        - (long) (csound->ekr * *p->dettack));
+      p->kloop = (int) ((int32) (p->h.insdshead->offtim * CS_EKR)
+                        - (int32) (CS_EKR * *p->dettack));
     }
     return OK;
 }
@@ -802,7 +852,9 @@ int brassset(CSOUND *csound, BRASS *p)
 int brass(CSOUND *csound, BRASS *p)
 {
     MYFLT *ar = p->ar;
-    int n, nsmps = csound->ksmps;
+    uint32_t offset = p->h.insdshead->ksmps_offset;
+    uint32_t early  = p->h.insdshead->ksmps_no_end;
+    uint32_t n, nsmps = CS_KSMPS;
     MYFLT amp = (*p->amp)*AMP_RSCALE; /* Normalise */
     MYFLT maxPressure = p->maxPressure = amp;
     int v_len = (int)p->vibr->flen;
@@ -820,7 +872,11 @@ int brass(CSOUND *csound, BRASS *p)
     }
     if (p->frq != *p->frequency) {             /* Set frequency if changed */
       p->frq = *p->frequency;
-      p->slideTarget = (csound->esr / p->frq * FL(2.0)) + FL(3.0);
+      if (p->limit > p->frq) {
+        p->frq =p->limit;
+        csound->Warning(csound, Str("frequency too low, set to minimum"));
+      }
+      p->slideTarget = (CS_ESR / p->frq * FL(2.0)) + FL(3.0);
                         /* fudge correction for filter delays */
        /*  we'll play a harmonic */
       if (DLineA_setDelay(csound, &p->delayLine, p->slideTarget)) return OK;
@@ -833,7 +889,12 @@ int brass(CSOUND *csound, BRASS *p)
                       p->lipTarget * (MYFLT)pow(4.0,(2.0* p->lipT) -1.0));
     }
 
-    for (n=0;n<nsmps;n++) {
+    if (UNLIKELY(offset)) memset(ar, '\0', offset*sizeof(MYFLT));
+    if (UNLIKELY(early)) {
+      nsmps -= early;
+      memset(&ar[nsmps], '\0', early*sizeof(MYFLT));
+    }
+    for (n=offset;n<nsmps;n++) {
       MYFLT     breathPressure;
       MYFLT     lastOutput;
       int       temp;
@@ -915,28 +976,29 @@ int shaker(void*,void*);
 int bowedbarset(void*,void*);
 int bowedbar(void*,void*);
 
-static OENTRY localops[] = {
-{ "wgclar",  S(CLARIN),5, "a", "kkkiikkkio",(SUBR)clarinset,NULL,   (SUBR)clarin},
-{ "wgflute", S(FLUTE), 5, "a", "kkkiikkkiovv",(SUBR)fluteset,NULL,  (SUBR)flute },
-{ "wgbow",   S(BOWED), 5, "a", "kkkkkkio", (SUBR)bowedset, NULL,    (SUBR)bowed },
-{ "wgbrass", S(BRASS), 5, "a", "kkkikkio", (SUBR)brassset, NULL,     (SUBR)brass},
-{ "mandol", S(MANDOL), 5, "a", "kkkkkkio", (SUBR)mandolinset, NULL,(SUBR)mandolin},
-{ "voice", S(VOICF),   5, "a", "kkkkkkii", (SUBR)voicformset, NULL,(SUBR)voicform},
-{ "fmbell",  S(FM4OP), 5, "a", "kkkkkkiiiii",
+static OENTRY physmod_localops[] = {
+{ "wgclar",  S(CLARIN),TR, 5, "a", "kkkiikkkjo",(SUBR)clarinset,NULL, (SUBR)clarin },
+{ "wgflute", S(FLUTE), TR, 5, "a", "kkkiikkkjovv",(SUBR)fluteset,NULL, (SUBR)flute },
+{ "wgbow",   S(BOWED), TR, 5, "a", "kkkkkkjo", (SUBR)bowedset, NULL,   (SUBR)bowed },
+{ "wgbrass", S(BRASS), TR, 5, "a", "kkkikkjo", (SUBR)brassset, NULL,    (SUBR)brass},
+{ "mandol", S(MANDOL), TR, 5, "a", "kkkkkkio",(SUBR)mandolinset,NULL,(SUBR)mandolin},
+{ "voice", S(VOICF),   TR, 5, "a", "kkkkkkii",(SUBR)voicformset,NULL,(SUBR)voicform},
+{ "fmbell",  S(FM4OP), TR, 5, "a", "kkkkkkjjjjjo",
                                             (SUBR)tubebellset,NULL,(SUBR)tubebell},
-{ "fmrhode", S(FM4OP), 5, "a", "kkkkkkiiiii",(SUBR)rhodeset,NULL,  (SUBR)tubebell},
-{ "fmwurlie", S(FM4OP),5, "a", "kkkkkkiiiii",(SUBR)wurleyset, NULL,(SUBR) wurley },
-{ "fmmetal", S(FM4OP), 5, "a", "kkkkkkiiiii",
+{ "fmrhode", S(FM4OP), TR, 5, "a", "kkkkkkiiiii",(SUBR)rhodeset,NULL,(SUBR)tubebell},
+{ "fmwurlie", S(FM4OP),TR, 5, "a", "kkkkkkiiiii",(SUBR)wurleyset,NULL,(SUBR)wurley },
+{ "fmmetal", S(FM4OP), TR, 5, "a", "kkkkkkiiiii",
                                           (SUBR)heavymetset, NULL, (SUBR)heavymet},
-{ "fmb3", S(FM4OP),    5, "a", "kkkkkkiiiii", (SUBR)b3set, NULL, (SUBR)hammondB3 },
-{ "fmvoice", S(FM4OPV),5, "a", "kkkkkkiiiii",(SUBR)FMVoiceset,NULL,(SUBR)FMVoice},
-{ "fmpercfl", S(FM4OP),5, "a", "kkkkkkiiiii",
+{ "fmb3", S(FM4OP),    TR, 5, "a", "kkkkkkjjjjj", (SUBR)b3set,NULL,(SUBR)hammondB3 },
+{ "fmvoice", S(FM4OPV),TR, 5, "a", "kkkkkkjjjjj",
+                                       (SUBR)FMVoiceset,NULL,(SUBR)FMVoice},
+{ "fmpercfl", S(FM4OP),TR, 5, "a", "kkkkkkjjjjj",
                                         (SUBR)percfluteset, NULL, (SUBR)percflute},
-{ "moog", S(MOOG1),    5, "a", "kkkkkkiii", (SUBR)Moog1set, NULL, (SUBR)Moog1  },
-{ "shaker", S(SHAKER), 5, "a", "kkkkko",  (SUBR)shakerset, NULL,   (SUBR)shaker},
-{ "wgbowedbar", S(BOWEDBAR), 5, "a","kkkkkoooo",
+{ "moog", S(MOOG1),    TR, 5, "a", "kkkkkkiii", (SUBR)Moog1set, NULL, (SUBR)Moog1  },
+{ "shaker", S(SHAKER), 0, 5, "a", "kkkkko",  (SUBR)shakerset, NULL,   (SUBR)shaker},
+{ "wgbowedbar", S(BOWEDBAR), 0, 5, "a","kkkkkoooo",
                              (SUBR)bowedbarset, NULL,(SUBR) bowedbar },
 };
 
-LINKAGE
+LINKAGE_BUILTIN(physmod_localops)
 
