@@ -30,17 +30,17 @@
 #include <math.h>
 
 #define WLN   1         /* time window is WLN*2*ksmps long */
-#define OPWLEN (2*WLN*csound->ksmps)    /* manifest used for final time wdw */
+#define OPWLEN (2*WLN*CS_KSMPS)    /* manifest used for final time wdw */
 
 /************************************************************/
 /*************PVBUFREAD**************************************/
 /************************************************************/
 
-int pvbufreadset(CSOUND *csound, PVBUFREAD *p)
+int pvbufreadset_(CSOUND *csound, PVBUFREAD *p, int stringname)
 {
     char     pvfilnam[MAXNAME];
     PVOCEX_MEMFILE  pp;
-    int      frInc, chans, size; /* THESE SHOULD BE SAVED IN PVOC STRUCT */
+    int      frInc, chans; /* THESE SHOULD BE SAVED IN PVOC STRUCT */
 
     {
       PVOC_GLOBALS  *p_ = PVOC_GetGlobals(csound);
@@ -58,8 +58,14 @@ int pvbufreadset(CSOUND *csound, PVBUFREAD *p)
       p->fftBuf = fltp;   /* fltp += PVFFTSIZE; */  /* Not needed */
     }
 
-    csound->strarg2name(csound, pvfilnam, p->ifilno, "pvoc.", p->XSTRCODE);
-    if (csound->PVOCEX_LoadFile(csound, pvfilnam, &pp) != 0)
+    if (stringname==0){
+      if (csound->ISSTRCOD(*p->ifilno))
+        strncpy(pvfilnam,get_arg_string(csound, *p->ifilno), MAXNAME-1);
+      else csound->strarg2name(csound, pvfilnam, p->ifilno, "pvoc.",0);
+    }
+    else strncpy(pvfilnam, ((STRINGDAT *)p->ifilno)->data, MAXNAME-1);
+
+    if (UNLIKELY(csound->PVOCEX_LoadFile(csound, pvfilnam, &pp) != 0))
       return csound->InitError(csound, Str("PVBUFREAD cannot load %s"),
                                        pvfilnam);
 
@@ -67,9 +73,9 @@ int pvbufreadset(CSOUND *csound, PVBUFREAD *p)
     frInc    = pp.overlap;
     chans    = pp.chans;
     p->asr   = pp.srate;
-    if (p->asr != csound->esr) {                /* & chk the data */
+    if (p->asr != CS_ESR) {                /* & chk the data */
       csound->Warning(csound, Str("%s's srate = %8.0f, orch's srate = %8.0f"),
-                              pvfilnam, p->asr, csound->esr);
+                              pvfilnam, p->asr, CS_ESR);
     }
     if (p->frSiz > PVFRAMSIZE) {
       return csound->InitError(csound,
@@ -87,9 +93,8 @@ int pvbufreadset(CSOUND *csound, PVBUFREAD *p)
     }
     p->frPtr = (float*) pp.data;
     p->maxFr = pp.nframes - 1;
-    p->frPktim = (MYFLT) csound->ksmps / (MYFLT) frInc;
-    p->frPrtim = csound->esr / (MYFLT) frInc;
-    size = pvfrsiz(p);          /* size used in def of OPWLEN ? */
+    p->frPktim = (MYFLT) CS_KSMPS / (MYFLT) frInc;
+    p->frPrtim = CS_ESR / (MYFLT) frInc;
     p->prFlg = 1;       /* true */
     /* amplitude scale for PVOC */
  /* p->scale = (MYFLT) pp.fftsize * ((MYFLT) pp.fftsize / (MYFLT) pp.winsize);
@@ -97,28 +102,32 @@ int pvbufreadset(CSOUND *csound, PVBUFREAD *p)
     p->scale = (MYFLT) pp.fftsize * FL(0.5);
     p->scale *= csound->GetInverseRealFFTScale(csound, pp.fftsize);
 
-    if ((OPWLEN / 2 + 1) > PVWINLEN ) {
+    if (UNLIKELY((OPWLEN / 2 + 1) > PVWINLEN )) {
       return csound->InitError(csound, Str("ksmps of %d needs wdw of %d, "
                                            "max is %d for pv %s"),
-                                       csound->ksmps, (int) (OPWLEN / 2 + 1),
+                                       CS_KSMPS, (int) (OPWLEN / 2 + 1),
                                        (int) PVWINLEN, pvfilnam);
     }
 
     return OK;
 }
 
+int pvbufreadset(CSOUND *csound, PVBUFREAD *p){
+  return pvbufreadset_(csound,p,0);
+}
+
+int pvbufreadset_S(CSOUND *csound, PVBUFREAD *p){
+  return pvbufreadset_(csound,p,1);
+}
+
 int pvbufread(CSOUND *csound, PVBUFREAD *p)
 {
     MYFLT  frIndx;
     MYFLT  *buf = p->fftBuf;
     int    size = pvfrsiz(p);
 
-    if (p->auxch.auxp == NULL) {        /* RWD fix */
-      return csound->PerfError(csound, Str("pvbufread: not initialised"));
-    }
-    if ((frIndx = *p->ktimpnt * p->frPrtim) < 0) {
-      return csound->PerfError(csound, Str("PVOC timpnt < 0"));
-    }
+    if (UNLIKELY(p->auxch.auxp == NULL)) goto err1;        /* RWD fix */
+    if (UNLIKELY((frIndx = *p->ktimpnt * p->frPrtim) < 0)) goto err2;
     if (frIndx > (MYFLT) p->maxFr) {    /* not past last one */
       frIndx = (MYFLT) p->maxFr;
       if (p->prFlg) {
@@ -130,21 +139,26 @@ int pvbufread(CSOUND *csound, PVBUFREAD *p)
     p->buf = buf;
 
     return OK;
+ err1:
+    return csound->PerfError(csound, p->h.insdshead,
+                             Str("pvbufread: not initialised"));
+ err2:
+    return csound->PerfError(csound, p->h.insdshead, Str("PVOC timpnt < 0"));
 }
 
 /************************************************************/
 /*************PVINTERP**************************************/
 /************************************************************/
-int pvinterpset(CSOUND *csound, PVINTERP *p)
+int pvinterpset_(CSOUND *csound, PVINTERP *p, int stringname)
 {
-    int      i;
+    unsigned int      i;
     char     pvfilnam[MAXNAME];
     PVOCEX_MEMFILE  pp;
-    int      frInc, chans, size; /* THESE SHOULD BE SAVED IN PVOC STRUCT */
+    int      frInc, chans; /* THESE SHOULD BE SAVED IN PVOC STRUCT */
 
     p->pp = PVOC_GetGlobals(csound);
     p->pvbufread = p->pp->pvbufreadaddr;
-    if (p->pvbufread == NULL)
+    if (UNLIKELY(p->pvbufread == NULL))
       return csound->InitError(csound,
                                Str("pvinterp: associated pvbufread not found"));
 
@@ -161,8 +175,13 @@ int pvinterpset(CSOUND *csound, PVINTERP *p)
       p->window = fltp;
     }
 
-    csound->strarg2name(csound, pvfilnam, p->ifilno, "pvoc.", p->XSTRCODE);
-    if (csound->PVOCEX_LoadFile(csound, pvfilnam, &pp) != 0)
+    if (stringname==0){
+      if (csound->ISSTRCOD(*p->ifilno))
+        strncpy(pvfilnam,get_arg_string(csound, *p->ifilno), MAXNAME-1);
+      else csound->strarg2name(csound, pvfilnam, p->ifilno, "pvoc.",0);
+    }
+    else strncpy(pvfilnam, ((STRINGDAT *)p->ifilno)->data, MAXNAME-1);;
+    if (UNLIKELY(csound->PVOCEX_LoadFile(csound, pvfilnam, &pp) != 0))
       return csound->InitError(csound, Str("PVINTERP cannot load %s"),
                                        pvfilnam);
 
@@ -170,17 +189,17 @@ int pvinterpset(CSOUND *csound, PVINTERP *p)
     frInc    = pp.overlap;
     chans    = pp.chans;
     p->asr   = pp.srate;
-    if (p->asr != csound->esr) {                /* & chk the data */
+    if (p->asr != CS_ESR) {                /* & chk the data */
       csound->Warning(csound, Str("%s's srate = %8.0f, orch's srate = %8.0f"),
-                              pvfilnam, p->asr, csound->esr);
+                              pvfilnam, p->asr, CS_ESR);
     }
-    if (p->frSiz != p->pvbufread->frSiz) {
+    if (UNLIKELY(p->frSiz != p->pvbufread->frSiz)) {
       return csound->InitError(csound,
                                Str("pvinterp: %s: frame size %d does not "
                                    "match pvbufread frame size %d\n"), pvfilnam,
                                (int) p->frSiz, (int) p->pvbufread->frSiz);
     }
-    if (chans != 1) {
+    if (UNLIKELY(chans != 1)) {
       return csound->InitError(csound, Str("%d chans (not 1) in PVOC file %s"),
                                        (int) chans, pvfilnam);
     }
@@ -189,11 +208,10 @@ int pvinterpset(CSOUND *csound, PVINTERP *p)
     p->baseFr = 0;  /* point to first data frame */
     p->maxFr = pp.nframes - 1;
     /* highest possible frame index */
-    p->frPktim = (MYFLT) csound->ksmps / (MYFLT) frInc;
+    p->frPktim = (MYFLT) CS_KSMPS / (MYFLT) frInc;
     /* factor by which to mult expand phase diffs (ratio of samp spacings) */
-    p->frPrtim = csound->esr / (MYFLT) frInc;
+    p->frPrtim = CS_ESR / (MYFLT) frInc;
     /* factor by which to mulitply 'real' time index to get frame index */
-    size = pvfrsiz(p);          /* size used in def of OPWLEN ? */
     /* amplitude scale for PVOC */
  /* p->scale = (MYFLT) pp.fftsize * ((MYFLT) pp.fftsize / (MYFLT) pp.winsize);
   */
@@ -208,22 +226,32 @@ int pvinterpset(CSOUND *csound, PVINTERP *p)
     for (i = 0; i < pvdasiz(p); ++i) {      /* or maybe pvdasiz(p) */
       p->lastPhase[i] = FL(0.0);
     }
-    if ((OPWLEN / 2 + 1) > PVWINLEN) {
+    if (UNLIKELY((OPWLEN / 2 + 1) > PVWINLEN)) {
       return csound->InitError(csound, Str("ksmps of %d needs wdw of %d, "
                                            "max is %d for pv %s"),
-                                       csound->ksmps, (OPWLEN / 2 + 1),
+                                       CS_KSMPS, (OPWLEN / 2 + 1),
                                        PVWINLEN, pvfilnam);
     }
     for (i = 0; i < OPWLEN / 2 + 1; ++i)    /* time window is OPWLEN long */
       p->window[i] = (MYFLT) (0.5 - 0.5 * cos(TWOPI*(double)i/(double)OPWLEN));
     /* NB: HANNING */
-    for (i = 0; i< pvfrsiz(p); ++i)
-      p->outBuf[i] = FL(0.0);
+    memset(p->outBuf, 0, pvfrsiz(p)*sizeof(MYFLT));
+    /* for (i = 0; i< pvfrsiz(p); ++i) */
+    /*   p->outBuf[i] = FL(0.0); */
     MakeSinc(p->pp);                    /* sinctab is same for all instances */
 
     return OK;
 }
 
+int pvinterpset(CSOUND *csound, PVINTERP *p){
+  return pvinterpset_(csound,p,0);
+}
+
+int pvinterpset_S(CSOUND *csound, PVINTERP *p){
+  return pvinterpset_(csound,p,1);
+}
+
+
 int pvinterp(CSOUND *csound, PVINTERP *p)
 {
     MYFLT   *ar = p->rslt;
@@ -236,25 +264,19 @@ int pvinterp(CSOUND *csound, PVINTERP *p)
     int     circBufSize = PVFFTSIZE;
     MYFLT   pex, scaleFac = p->scale;
     PVBUFREAD *q = p->pvbufread;
-    long    i, j;
+    int32    i, j;
 
-    if (p->auxch.auxp == NULL) {    /* RWD Fix */
-      return csound->PerfError(csound, Str("pvinterp: not initialised"));
-    }
+    if (UNLIKELY(p->auxch.auxp == NULL)) goto err1;    /* RWD Fix */
     pex = *p->kfmod;
     outlen = (int) (((MYFLT) size) / pex);
     /* use outlen to check window/krate/transpose combinations */
-    if (outlen>PVFFTSIZE) { /* Maximum transposition down is one octave */
-                            /* ..so we won't run into buf2Size problems */
-      return csound->PerfError(csound, Str("PVOC transpose too low"));
-    }
-    if (outlen<2*csound->ksmps) {   /* minimum post-squeeze windowlength */
-      return csound->PerfError(csound, Str("PVOC transpose too high"));
-    }
+    if (UNLIKELY(outlen>PVFFTSIZE))  /* Maximum transposition down is one octave */
+                           /* ..so we won't run into buf2Size problems */
+      goto err2;
+    if (UNLIKELY(outlen<(int)(2*CS_KSMPS)))
+      goto err3;   /* minimum post-squeeze windowlength */
     buf2Size = OPWLEN;     /* always window to same length after DS */
-    if ((frIndx = *p->ktimpnt * p->frPrtim) < 0) {
-      return csound->PerfError(csound, Str("PVOC timpnt < 0"));
-    }
+    if (UNLIKELY((frIndx = *p->ktimpnt * p->frPrtim) < 0)) goto err4;
     if (frIndx > (MYFLT)p->maxFr) { /* not past last one */
       frIndx = (MYFLT)p->maxFr;
       if (p->prFlg) {
@@ -276,7 +298,7 @@ int pvinterp(CSOUND *csound, PVINTERP *p)
       buf[j] = (buf[j] + ((q->buf[j] - buf[j]) * *p->kfreqinterp));
     }
 /*******************************************************************/
-    FrqToPhase(buf, asize, pex * (MYFLT) csound->ksmps, p->asr,
+    FrqToPhase(buf, asize, pex * (MYFLT) CS_KSMPS, p->asr,
                (MYFLT) (0.5 * ((pex / p->lastPex) - 1)));
     /* accumulate phase and wrap to range -PI to PI */
     RewrapPhase(buf, asize, p->lastPhase);
@@ -292,31 +314,42 @@ int pvinterp(CSOUND *csound, PVINTERP *p)
              sizeof(MYFLT) * buf2Size);
     ApplyHalfWin(buf2, p->window, buf2Size);
 
-    addToCircBuf(buf2, p->outBuf, p->opBpos, csound->ksmps, circBufSize);
-    writeClrFromCircBuf(p->outBuf, ar, p->opBpos, csound->ksmps, circBufSize);
-    p->opBpos += csound->ksmps;
+    addToCircBuf(buf2, p->outBuf, p->opBpos, CS_KSMPS, circBufSize);
+    writeClrFromCircBuf(p->outBuf, ar, p->opBpos, CS_KSMPS, circBufSize);
+    p->opBpos += CS_KSMPS;
     if (p->opBpos > circBufSize)
       p->opBpos -= circBufSize;
-    addToCircBuf(buf2 + csound->ksmps, p->outBuf, p->opBpos,
-                 buf2Size - csound->ksmps, circBufSize);
+    addToCircBuf(buf2 + CS_KSMPS, p->outBuf, p->opBpos,
+                 buf2Size - CS_KSMPS, circBufSize);
     p->lastPex = pex;        /* needs to know last pitchexp to update phase */
 
     return OK;
+ err1:
+    return csound->PerfError(csound, p->h.insdshead,
+                             Str("pvinterp: not initialised"));
+ err2:
+    return csound->PerfError(csound, p->h.insdshead,
+                             Str("PVOC transpose too low"));
+ err3:
+    return csound->PerfError(csound, p->h.insdshead,
+                             Str("PVOC transpose too high"));
+ err4:
+    return csound->PerfError(csound, p->h.insdshead, Str("PVOC timpnt < 0"));
 }
 
 /************************************************************/
 /************* PVCROSS **************************************/
 /************************************************************/
-int pvcrossset(CSOUND *csound, PVCROSS *p)
+int pvcrossset_(CSOUND *csound, PVCROSS *p, int stringname)
 {
-    int      i;
+    uint32_t i;
     char     pvfilnam[MAXNAME];
     PVOCEX_MEMFILE  pp;
-    int      frInc, chans, size; /* THESE SHOULD BE SAVED IN PVOC STRUCT */
+    int      frInc, chans; /* THESE SHOULD BE SAVED IN PVOC STRUCT */
 
     p->pp = PVOC_GetGlobals(csound);
     p->pvbufread = p->pp->pvbufreadaddr;
-    if (p->pvbufread == NULL)
+    if (UNLIKELY(p->pvbufread == NULL))
       return csound->InitError(csound,
                                Str("pvcross: associated pvbufread not found"));
 
@@ -332,25 +365,31 @@ int pvcrossset(CSOUND *csound, PVCROSS *p)
       p->outBuf = fltp;      fltp += PVFFTSIZE;
       p->window = fltp;
     }
-    csound->strarg2name(csound, pvfilnam, p->ifilno, "pvoc.", p->XSTRCODE);
-    if (csound->PVOCEX_LoadFile(csound, pvfilnam, &pp) != 0)
+    if (stringname==0){
+      if (csound->ISSTRCOD(*p->ifilno))
+        strncpy(pvfilnam,get_arg_string(csound, *p->ifilno), MAXNAME-1);
+      else csound->strarg2name(csound, pvfilnam, p->ifilno, "pvoc.",0);
+    }
+    else strncpy(pvfilnam, ((STRINGDAT *)p->ifilno)->data, MAXNAME-1);
+
+    if (UNLIKELY(csound->PVOCEX_LoadFile(csound, pvfilnam, &pp) != 0))
       return csound->InitError(csound, Str("PVCROSS cannot load %s"), pvfilnam);
 
     p->frSiz = pp.fftsize;
     frInc    = pp.overlap;
     chans    = pp.chans;
     p->asr   = pp.srate;
-    if (p->asr != csound->esr) {                /* & chk the data */
+    if (p->asr != CS_ESR) {                /* & chk the data */
       csound->Warning(csound, Str("%s's srate = %8.0f, orch's srate = %8.0f"),
-                              pvfilnam, p->asr, csound->esr);
+                              pvfilnam, p->asr, CS_ESR);
     }
-    if (p->frSiz != p->pvbufread->frSiz) {
+    if (UNLIKELY(p->frSiz != p->pvbufread->frSiz)) {
       return csound->InitError(csound,
                                Str("pvcross: %s: frame size %d does not "
                                    "match pvbufread frame size %d\n"), pvfilnam,
                                (int) p->frSiz, (int) p->pvbufread->frSiz);
     }
-    if (chans != 1) {
+    if (UNLIKELY(chans != 1)) {
       return csound->InitError(csound, Str("%d chans (not 1) in PVOC file %s"),
                                        (int) chans, pvfilnam);
     }
@@ -359,11 +398,10 @@ int pvcrossset(CSOUND *csound, PVCROSS *p)
     p->baseFr = 0;  /* point to first data frame */
     p->maxFr = pp.nframes - 1;
     /* highest possible frame index */
-    p->frPktim = (MYFLT) csound->ksmps / (MYFLT) frInc;
+    p->frPktim = (MYFLT) CS_KSMPS / (MYFLT) frInc;
     /* factor by which to mult expand phase diffs (ratio of samp spacings) */
-    p->frPrtim = csound->esr / (MYFLT) frInc;
+    p->frPrtim = CS_ESR / (MYFLT) frInc;
     /* factor by which to mulitply 'real' time index to get frame index */
-    size = pvfrsiz(p);          /* size used in def of OPWLEN ? */
     /* amplitude scale for PVOC */
  /* p->scale = (MYFLT) pp.fftsize * ((MYFLT) pp.fftsize / (MYFLT) pp.winsize);
   */
@@ -376,25 +414,34 @@ int pvcrossset(CSOUND *csound, PVCROSS *p)
     for (i = 0; i < pvdasiz(p); ++i) {      /* or maybe pvdasiz(p) */
         p->lastPhase[i] = FL(0.0);
     }
-    if ((OPWLEN / 2 + 1) > PVWINLEN ) {
+    if (UNLIKELY((OPWLEN / 2 + 1) > PVWINLEN )) {
       return csound->InitError(csound, Str("ksmps of %d needs wdw of %d, "
                                            "max is %d for pv %s"),
-                                       csound->ksmps, (OPWLEN / 2 + 1),
+                                       CS_KSMPS, (OPWLEN / 2 + 1),
                                        PVWINLEN, pvfilnam);
     }
     for (i = 0; i < OPWLEN / 2 + 1; ++i)    /* time window is OPWLEN long */
       p->window[i] = (MYFLT) (0.5 - 0.5 * cos(TWOPI*(double)i/(double)OPWLEN));
     /* NB: HANNING */
-    for (i = 0; i < pvfrsiz(p); ++i)
-      p->outBuf[i] = FL(0.0);
+    memset(p->outBuf, 0, pvfrsiz(p)*sizeof(MYFLT));
+    /* for (i = 0; i < pvfrsiz(p); ++i) */
+    /*   p->outBuf[i] = FL(0.0); */
     MakeSinc(p->pp);                    /* sinctab is same for all instances */
-
+    if (p->memenv.auxp == NULL || p->memenv.size < pvdasiz(p)*sizeof(MYFLT))
+      csound->AuxAlloc(csound, pvdasiz(p) * sizeof(MYFLT), &p->memenv);
     return OK;
 }
 
+int pvcrossset(CSOUND *csound, PVCROSS *p){
+  return pvcrossset_(csound,p,0);
+}
+
+int pvcrossset_S(CSOUND *csound, PVCROSS *p) {
+return pvcrossset_(csound,p,1);
+}
+
 int pvcross(CSOUND *csound, PVCROSS *p)
 {
-    int     n;
     MYFLT   *ar = p->rslt;
     MYFLT   frIndx;
     MYFLT   *buf = p->fftBuf;
@@ -406,27 +453,21 @@ int pvcross(CSOUND *csound, PVCROSS *p)
     int     specwp = (int) *p->ispecwp; /* spectral warping flag */
     MYFLT   pex, scaleFac = p->scale;
     PVBUFREAD *q = p->pvbufread;
-    long    i, j;
+    int32   i, j;
     MYFLT   ampscale1 = *p->kampscale1;
     MYFLT   ampscale2 = *p->kampscale2;
 
-    if (p->auxch.auxp == NULL) {        /* RWD Fix */
-      return csound->PerfError(csound, Str("pvcross: not initialised"));
-    }
+    if (UNLIKELY(p->auxch.auxp == NULL)) goto err1;        /* RWD Fix */
     pex = *p->kfmod;
     outlen = (int) (((MYFLT) size) / pex);
     /* use outlen to check window/krate/transpose combinations */
-    if (outlen>PVFFTSIZE) { /* Maximum transposition down is one octave */
-                            /* ..so we won't run into buf2Size problems */
-      return csound->PerfError(csound, Str("PVOC transpose too low"));
-    }
-    if (outlen<2*csound->ksmps) {  /* minimum post-squeeze windowlength */
-      return csound->PerfError(csound, Str("PVOC transpose too high"));
-    }
+    if (UNLIKELY(outlen>PVFFTSIZE)) /* Maximum transposition down is one octave */
+                                    /* ..so we won't run into buf2Size problems */
+      goto err2;
+    if (UNLIKELY(outlen<(int)(2*CS_KSMPS))) /* minimum post-squeeze windowlength */
+      goto err3;
     buf2Size = OPWLEN;     /* always window to same length after DS */
-    if ((frIndx = *p->ktimpnt * p->frPrtim) < 0) {
-      return csound->PerfError(csound, Str("PVOC timpnt < 0"));
-    }
+    if ((frIndx = *p->ktimpnt * p->frPrtim) < 0) goto err4;
     if (frIndx > (MYFLT) p->maxFr) {    /* not past last one */
       frIndx = (MYFLT) p->maxFr;
       if (p->prFlg) {
@@ -444,7 +485,7 @@ int pvcross(CSOUND *csound, PVCROSS *p)
       buf[i] = ((buf[i] * ampscale2) + (q->buf[i] * ampscale1)) * scaleFac;
 /***************************************************/
 
-    FrqToPhase(buf, asize, pex * (MYFLT) csound->ksmps, p->asr,
+    FrqToPhase(buf, asize, pex * (MYFLT) CS_KSMPS, p->asr,
                (MYFLT) (0.5 * ((pex / p->lastPex) - 1)));
     /* accumulate phase and wrap to range -PI to PI */
     RewrapPhase(buf, asize, p->lastPhase);
@@ -452,10 +493,12 @@ int pvcross(CSOUND *csound, PVCROSS *p)
     if (specwp == 0 || (p->prFlg)++ == -(int) specwp) {
       /* ?screws up when prFlg used */
       /* specwp=0 => normal; specwp = -n => just nth frame */
-      if (specwp < 0)
+#ifdef BETA
+     if (specwp < 0)
         csound->Message(csound, Str("PVOC debug: one frame gets through\n"));
+#endif
       if (specwp > 0)
-        PreWarpSpec(p->pp, buf, asize, pex);
+        PreWarpSpec(buf, asize, pex, (MYFLT *)p->memenv.auxp);
 
       Polar2Real_PVOC(csound, buf, (int) size);
 
@@ -470,19 +513,29 @@ int pvcross(CSOUND *csound, PVCROSS *p)
         ApplyHalfWin(buf2, p->window, buf2Size);
     }
     else {
-      for (n = 0; n < buf2Size; ++n)
-        buf2[n] = FL(0.0);
+      memset(buf2, 0, buf2Size*sizeof(MYFLT));
     }
 
-    addToCircBuf(buf2, p->outBuf, p->opBpos, csound->ksmps, circBufSize);
-    writeClrFromCircBuf(p->outBuf, ar, p->opBpos, csound->ksmps, circBufSize);
-    p->opBpos += csound->ksmps;
+    addToCircBuf(buf2, p->outBuf, p->opBpos, CS_KSMPS, circBufSize);
+    writeClrFromCircBuf(p->outBuf, ar, p->opBpos, CS_KSMPS, circBufSize);
+    p->opBpos += CS_KSMPS;
     if (p->opBpos > circBufSize)
       p->opBpos -= circBufSize;
-    addToCircBuf(buf2 + csound->ksmps, p->outBuf, p->opBpos,
-                 buf2Size - csound->ksmps, circBufSize);
+    addToCircBuf(buf2 + CS_KSMPS, p->outBuf, p->opBpos,
+                 buf2Size - CS_KSMPS, circBufSize);
     p->lastPex = pex;       /* needs to know last pitchexp to update phase */
 
     return OK;
+ err1:
+    return csound->PerfError(csound, p->h.insdshead,
+                             Str("pvcross: not initialised"));
+ err2:
+    return csound->PerfError(csound, p->h.insdshead,
+                             Str("PVOC transpose too low"));
+ err3:
+    return csound->PerfError(csound, p->h.insdshead,
+                             Str("PVOC transpose too high"));
+ err4:
+    return csound->PerfError(csound, p->h.insdshead,
+                             Str("PVOC timpnt < 0"));
 }
-
