@@ -22,8 +22,27 @@
 */
 
 #include "csdl.h"
+#include "interlocks.h"
 #include "pstream.h"
 
+static int STR_ARG_P(CSOUND *csound, void* arg) {
+    CS_TYPE *cs_type = csound->GetTypeForArg(arg);
+    if(strcmp("S", cs_type->varTypeName) == 0) {
+        return 1;
+    } else {
+        return 0;
+    }
+}
+
+static int ASIG_ARG_P(CSOUND *csound, void* arg) {
+    CS_TYPE *cs_type = csound->GetTypeForArg(arg);
+    if(strcmp("a", cs_type->varTypeName) == 0) {
+        return 1;
+    } else {
+        return 0;
+    }
+}
+
 typedef struct CsoundArgStack_s CsoundArgStack_t;
 
 #define CS_STACK_ALIGN          8
@@ -71,20 +90,22 @@ typedef struct POP_OPCODE_ {
     int     initDone;
 } POP_OPCODE;
 
-/* fassign() was taken from pstream.c, written by Richard Dobson */
+/* fsg_assign() was taken from pstream.c, written by Richard Dobson */
 
-static CS_NOINLINE void fassign(CSOUND *csound,
+static CS_NOINLINE void fsg_assign(CSOUND *csound,
                                 PVSDAT *fdst, const PVSDAT *fsrc)
 {
-    if (fsrc->frame.auxp == NULL)
-      csound->Die(csound, Str("fsig = : source signal is not initialised"));
+  if (UNLIKELY(fsrc->frame.auxp == NULL)) {
+      csound->ErrorMsg(csound, Str("fsig = : source signal is not initialised"));
+      return;
+  }
     fdst->N = fsrc->N;
     fdst->overlap = fsrc->overlap;
     fdst->winsize = fsrc->winsize;
     fdst->wintype = fsrc->wintype;
     fdst->format = fsrc->format;
     if (fdst->frame.auxp == NULL ||
-        fdst->frame.size != (fdst->N + 2L) * (long) sizeof(float))
+        fdst->frame.size != (uint32_t)((fdst->N + 2L) * sizeof(float)))
       csound->AuxAlloc(csound,
                        (fdst->N + 2L) * (long) sizeof(float), &(fdst->frame));
     if (fdst->framecount != fsrc->framecount) {
@@ -104,51 +125,34 @@ static CS_NOINLINE int csoundStack_Error(void *p, const char *msg)
     CSOUND  *csound;
 
     csound = ((OPDS*) p)->insdshead->csound;
-    if (csound->ids != NULL && csound->pds == NULL) {
-      csound->InitError(csound, "%s: %s", csound->GetOpcodeName(p), msg);
-      csound->LongJmp(csound, CSOUND_INITIALIZATION);
-    }
-    else if (csound->ids == NULL && csound->pds != NULL) {
-      csound->PerfError(csound, "%s: %s", csound->GetOpcodeName(p), msg);
-      csound->LongJmp(csound, CSOUND_PERFORMANCE);
-    }
-    else
-      csound->Die(csound, "%s: %s", csound->GetOpcodeName(p), msg);
-    /* this is actually never reached */
+    csound->ErrorMsg(csound, "%s: %s", csound->GetOpcodeName(p), msg);
+
     return NOTOK;
 }
 
 static CS_NOINLINE int csoundStack_OverflowError(void *p)
 {
-    CSOUND  *csound;
-
-    csound = ((OPDS*) p)->insdshead->csound;
+    /* CSOUND  *csound= ((OPDS*) p)->insdshead->csound; */
     return csoundStack_Error(p, Str("stack overflow"));
 }
 
 static CS_NOINLINE int csoundStack_EmptyError(void *p)
 {
-    CSOUND  *csound;
-
-    csound = ((OPDS*) p)->insdshead->csound;
+    /* CSOUND  *csound((OPDS*) p)->insdshead->csound; */
     return csoundStack_Error(p, Str("cannot pop from empty stack"));
 }
 
 static CS_NOINLINE int csoundStack_TypeError(void *p)
 {
-    CSOUND  *csound;
-
-    csound = ((OPDS*) p)->insdshead->csound;
+    /* CSOUND  *csound = ((OPDS*) p)->insdshead->csound; */
     return csoundStack_Error(p, Str("argument number or type mismatch"));
 }
 
-static CS_NOINLINE int csoundStack_LengthError(void *p)
-{
-    CSOUND  *csound;
-
-    csound = ((OPDS*) p)->insdshead->csound;
-    return csoundStack_Error(p, Str("string argument is too long"));
-}
+/* static CS_NOINLINE int csoundStack_LengthError(void *p) */
+/* { */
+/*     /\* CSOUND  *csound = ((OPDS*) p)->insdshead->csound; *\/ */
+/*     return csoundStack_Error(p, Str("string argument is too long")); */
+/* } */
 
 static CS_NOINLINE CsoundArgStack_t *csoundStack_AllocGlobals(CSOUND *csound,
                                                               int stackSize)
@@ -156,15 +160,18 @@ static CS_NOINLINE CsoundArgStack_t *csoundStack_AllocGlobals(CSOUND *csound,
     CsoundArgStack_t  *pp;
     int               nBytes;
 
-    if (stackSize < 1024)
+    if (UNLIKELY(stackSize < 1024))
       stackSize = 1024;
-    else if (stackSize > 16777200)
+    else if (UNLIKELY(stackSize > 16777200))
       stackSize = 16777200;
     nBytes = csoundStack_Align((int) sizeof(CsoundArgStack_t));
     nBytes += stackSize;
-    if (csound->CreateGlobalVariable(csound, "csArgStack", (size_t) nBytes)
-        != 0)
-      csound->Die(csound, Str("Error allocating argument stack"));
+    if (UNLIKELY(csound->CreateGlobalVariable(csound,
+                                              "csArgStack", (size_t) nBytes)
+                 != 0)) {
+      csound->ErrorMsg(csound, Str("Error allocating argument stack"));
+      return NULL;
+    }
     pp = (CsoundArgStack_t*) csound->QueryGlobalVariable(csound, "csArgStack");
     pp->curBundle = (CsoundArgStack_t*) NULL;
     pp->dataSpace =
@@ -186,35 +193,32 @@ static CS_NOINLINE CsoundArgStack_t *csoundStack_GetGlobals(CSOUND *csound)
     return pp;
 }
 
-static CS_NOINLINE int csoundStack_CreateArgMap(void *p, int *argMap,
+static CS_NOINLINE int csoundStack_CreateArgMap(PUSH_OPCODE *p, int *argMap,
                                                 int isOutput)
 {
     CSOUND  *csound;
-    int     i, argCnt, argCnt_i, argCnt_p, aMask, sMask, curOffs_i, curOffs_p;
+    int     i, argCnt, argCnt_i, argCnt_p, curOffs_i, curOffs_p;
+    MYFLT** args = p->args;
 
     csound = ((OPDS*) p)->insdshead->csound;
     if (!isOutput) {
       argCnt = csound->GetInputArgCnt(p);
-      aMask = csound->GetInputArgAMask(p);
-      sMask = csound->GetInputArgSMask(p);
     }
     else {
       argCnt = csound->GetOutputArgCnt(p);
-      aMask = csound->GetOutputArgAMask(p);
-      sMask = csound->GetOutputArgSMask(p);
     }
-    if (argCnt > 31)
+    if (UNLIKELY(argCnt > 31))
       return csoundStack_Error(p, Str("too many arguments"));
     argMap[0] = 0;
     argCnt_i = 0;
     argCnt_p = 0;
     for (i = 0; i < argCnt; i++) {
       int   maskVal = (1 << i);
-      if (aMask & maskVal) {
+      if (ASIG_ARG_P(csound, args[i])) {
         argMap[0] |= maskVal;
         argCnt_p++;
       }
-      else if (sMask & maskVal) {
+      else if (STR_ARG_P(csound, args[i])) {
         argCnt_i++;
       }
       else {
@@ -245,9 +249,9 @@ static CS_NOINLINE int csoundStack_CreateArgMap(void *p, int *argMap,
       int   maskVal = (1 << i);
       if (argMap[0] & maskVal) {
         /* performance time types */
-        if (aMask & maskVal) {
+        if (ASIG_ARG_P(csound, args[i])) {
           argMap[i + 3] = (curOffs_p | CS_STACK_A);
-          curOffs_p += ((int) sizeof(MYFLT) * csound->ksmps);
+          curOffs_p += ((int) sizeof(MYFLT) * CS_KSMPS);
         }
         else {
           argMap[i + 3] = (curOffs_p | CS_STACK_K);
@@ -256,10 +260,10 @@ static CS_NOINLINE int csoundStack_CreateArgMap(void *p, int *argMap,
       }
       else {
         /* init time types */
-        if (sMask & maskVal) {
+        if (STR_ARG_P(csound, args[i])) {
           argMap[i + 3] = (curOffs_i | CS_STACK_S);
-          curOffs_i += csound->strVarMaxLen;
-          curOffs_i = csoundStack_Align(curOffs_i);
+          curOffs_i += (int) sizeof(STRINGDAT);
+            /* curOffs_i = csoundStack_Align(curOffs_i);*/
         }
         else {
           argMap[i + 3] = (curOffs_i | CS_STACK_I);
@@ -282,7 +286,7 @@ static CS_NOINLINE int csoundStack_CreateArgMap(void *p, int *argMap,
 
 static int stack_opcode_init(CSOUND *csound, STACK_OPCODE *p)
 {
-    if (csound->QueryGlobalVariable(csound, "csArgStack") != NULL)
+    if (UNLIKELY(csound->QueryGlobalVariable(csound, "csArgStack") != NULL))
       return csound->InitError(csound, Str("the stack is already allocated"));
     csoundStack_AllocGlobals(csound, (int) (*(p->iStackSize) + 0.5));
     return OK;
@@ -290,8 +294,9 @@ static int stack_opcode_init(CSOUND *csound, STACK_OPCODE *p)
 
 static int notinit_opcode_stub_perf(CSOUND *csound, void *p)
 {
-    return csound->PerfError(csound, Str("%s: not initialised"),
-                                     csound->GetOpcodeName(p));
+    return csound->PerfError(csound, ((OPDS*)p)->insdshead,
+                             Str("%s: not initialised"),
+                             csound->GetOpcodeName(p));
 }
 
 static int push_opcode_perf(CSOUND *csound, PUSH_OPCODE *p)
@@ -318,11 +323,19 @@ static int push_opcode_perf(CSOUND *csound, PUSH_OPCODE *p)
           case CS_STACK_A:
             {
               MYFLT *src, *dst;
-              int   j;
+              uint32_t offset = p->h.insdshead->ksmps_offset;
+              uint32_t early  = p->h.insdshead->ksmps_no_end;
+              uint32_t nsmps = CS_KSMPS;
               src = p->args[i];
               dst = (MYFLT*) ((char*) bp + (int) (curOffs & (int) 0x00FFFFFF));
-              for (j = 0; j < csound->ksmps; j++)
-                dst[j] = src[j];
+              if (UNLIKELY(offset)) memset(dst, '\0', offset*sizeof(MYFLT));
+              if (UNLIKELY(early)) {
+                nsmps -= early;
+                memset(&dst[nsmps], '\0', early*sizeof(MYFLT));
+              }
+              memcpy(&dst[offset], &src[offset], sizeof(MYFLT)*(nsmps-offset));
+              //for (j = 0; j < nsmps; j++)
+              //dst[j] = src[j];
             }
           }
         }
@@ -336,7 +349,7 @@ static int push_opcode_init(CSOUND *csound, PUSH_OPCODE *p)
 {
     if (!p->initDone) {
       p->pp = csoundStack_GetGlobals(csound);
-      if (csoundStack_CreateArgMap(p, &(p->argMap[0]), 0) != OK)
+      if (UNLIKELY(csoundStack_CreateArgMap(p, (int*)&(p->argMap[0]), 0) != OK))
         return NOTOK;
       p->h.opadr = (int (*)(CSOUND *, void *)) push_opcode_perf;
       p->initDone = 1;
@@ -344,7 +357,8 @@ static int push_opcode_init(CSOUND *csound, PUSH_OPCODE *p)
     if (p->argMap[1] != 0) {
       void  *bp;
       int   i, *ofsp;
-      if (p->pp->freeSpaceOffset + p->argMap[1] > p->pp->freeSpaceEndOffset)
+      if (UNLIKELY(p->pp->freeSpaceOffset + p->argMap[1] >
+                   p->pp->freeSpaceEndOffset))
         return csoundStack_OverflowError(p);
       bp = (void*) ((char*) p->pp->dataSpace + (int) p->pp->freeSpaceOffset);
       p->pp->freeSpaceOffset += p->argMap[1];
@@ -362,19 +376,17 @@ static int push_opcode_init(CSOUND *csound, PUSH_OPCODE *p)
             break;
           case CS_STACK_S:
             {
-              char  *src, *dst;
-              int   j, maxLen;
-              src = (char*) p->args[i];
-              dst = (char*) bp + (int) (curOffs & (int) 0x00FFFFFF);
-              maxLen = csound->strVarMaxLen - 1;
-              for (j = 0; src[j] != (char) 0; j++) {
-                dst[j] = src[j];
-                if (j >= maxLen) {
-                  dst[j] = (char) 0;
-                  csoundStack_LengthError(p);
-                }
-              }
-              dst[j] = (char) 0;
+              char  *src;
+              STRINGDAT **ans, *dst;
+              /* int   j, maxLen; */
+              src = ((STRINGDAT*) p->args[i])->data;
+              ans = ((STRINGDAT**)(char*) bp + (int) (curOffs & (int) 0x00FFFFFF));
+              dst = (STRINGDAT*) csound->Malloc(csound, sizeof(STRINGDAT));
+              dst->data = csound->Strdup(csound, src);
+              dst->size = strlen(src) + 1;
+              *ans = dst;
+              /* printf("***dst = %p %p %d, \"%s\"\n", */
+              /*        ans, dst, dst->size, dst->data); */
             }
           }
         }
@@ -389,7 +401,7 @@ static int pop_opcode_perf(CSOUND *csound, POP_OPCODE *p)
     if (p->argMap[2] != 0) {
       void  *bp;
       int   i, *ofsp;
-      if (p->pp->curBundle == NULL)
+      if (UNLIKELY(p->pp->curBundle == NULL))
         return csoundStack_EmptyError(p);
       bp = p->pp->curBundle;
       ofsp = (int*) ((char*) bp + (int) csoundStack_Align((int) sizeof(void*)));
@@ -407,11 +419,19 @@ static int pop_opcode_perf(CSOUND *csound, POP_OPCODE *p)
           case CS_STACK_A:
             {
               MYFLT *src, *dst;
-              int   j;
+              uint32_t offset = p->h.insdshead->ksmps_offset;
+              uint32_t early  = p->h.insdshead->ksmps_no_end;
+              uint32_t nsmps = CS_KSMPS;
               src = (MYFLT*) ((char*) bp + (int) (curOffs & (int) 0x00FFFFFF));
               dst = p->args[i];
-              for (j = 0; j < csound->ksmps; j++)
-                dst[j] = src[j];
+              if (UNLIKELY(offset)) memset(dst, '\0', offset*sizeof(MYFLT));
+              if (UNLIKELY(early)) {
+                nsmps -= early;
+                memset(&dst[nsmps], '\0', early*sizeof(MYFLT));
+              }
+              memcpy(&dst[offset], &src[offset], (nsmps-offset)*sizeof(MYFLT));
+              //for (j = 0; j < CS_KSMPS; j++)
+              //  dst[j] = src[j];
             }
             break;
           }
@@ -427,7 +447,8 @@ static int pop_opcode_init(CSOUND *csound, POP_OPCODE *p)
 {
     if (!p->initDone) {
       p->pp = csoundStack_GetGlobals(csound);
-      if (csoundStack_CreateArgMap(p, &(p->argMap[0]), 1) != OK)
+      if (UNLIKELY(csoundStack_CreateArgMap((PUSH_OPCODE*)p,
+                                            &(p->argMap[0]), 1) != OK))
         return NOTOK;
       p->h.opadr = (int (*)(CSOUND *, void *)) pop_opcode_perf;
       p->initDone = 1;
@@ -451,8 +472,27 @@ static int pop_opcode_init(CSOUND *csound, POP_OPCODE *p)
                 *((MYFLT*) ((char*) bp + (int) (curOffs & (int) 0x00FFFFFF)));
             break;
           case CS_STACK_S:
-            strcpy((char*) p->args[i],
-                   (char*) bp + (int) (curOffs & (int) 0x00FFFFFF));
+            {
+              STRINGDAT **ans =
+                ((STRINGDAT**)(char*) bp + (int) (curOffs & (int) 0x00FFFFFF));
+              STRINGDAT *str = *ans;
+              STRINGDAT *dst = (STRINGDAT*)p->args[i];
+              /* printf("***string: %p\nbp=%p Off = %x\n", ans, bp, curOffs); */
+              /* printf("***string: %p->%s\n", str, dst->data); */
+              if (str==NULL)
+                return csound->InitError(csound, "pop of strings broken");
+              if (str->size>dst->size) {
+                csound->Free(csound,dst->data);
+                dst->data = csound->Strdup(csound, str->data);
+                dst->size = strlen(dst->data)+1;
+              }
+              else {
+                strcpy((char*) dst->data, str->data);
+              }
+              csound->Free(csound,str->data);
+              csound->Free(csound,str);
+              *ans = NULL;
+            }
             break;
           }
         }
@@ -469,7 +509,8 @@ static int push_f_opcode_perf(CSOUND *csound, PUSH_OPCODE *p)
     int     *ofsp;
     int     offs;
 
-    if (p->pp->freeSpaceOffset + p->argMap[2] > p->pp->freeSpaceEndOffset)
+    if (UNLIKELY(p->pp->freeSpaceOffset + p->argMap[2] >
+                 p->pp->freeSpaceEndOffset))
       return csoundStack_OverflowError(p);
     bp = (void*) ((char*) p->pp->dataSpace + (int) p->pp->freeSpaceOffset);
     p->pp->freeSpaceOffset += p->argMap[2];
@@ -511,7 +552,7 @@ static int push_f_opcode_init(CSOUND *csound, PUSH_OPCODE *p)
       p->h.opadr = (int (*)(CSOUND *, void *)) push_f_opcode_perf;
       p->initDone = 1;
     }
-    if (p->pp->freeSpaceOffset + p->argMap[1] > p->pp->freeSpaceEndOffset)
+    if (UNLIKELY(p->pp->freeSpaceOffset + p->argMap[1] > p->pp->freeSpaceEndOffset))
       return csoundStack_OverflowError(p);
     bp = (void*) ((char*) p->pp->dataSpace + (int) p->pp->freeSpaceOffset);
     p->pp->freeSpaceOffset += p->argMap[1];
@@ -533,17 +574,17 @@ static int pop_f_opcode_perf(CSOUND *csound, POP_OPCODE *p)
     int     *ofsp;
     int     offs;
 
-    if (p->pp->curBundle == NULL)
+    if (UNLIKELY(p->pp->curBundle == NULL))
       return csoundStack_EmptyError(p);
     bp = p->pp->curBundle;
     ofsp = (int*) ((char*) bp + (int) csoundStack_Align((int) sizeof(void*)));
     offs = p->argMap[3];
-    if (offs != *ofsp)
+    if (UNLIKELY(offs != *ofsp))
       csoundStack_TypeError(p);
     ofsp++;
-    if (*ofsp != CS_STACK_END)
+    if (UNLIKELY(*ofsp != CS_STACK_END))
       csoundStack_TypeError(p);
-    fassign(csound,
+    fsg_assign(csound,
             (PVSDAT*) p->args[0],
             *((PVSDAT**) ((char*) bp + (int) (offs & (int) 0x00FFFFFF))));
     p->pp->curBundle = *((void**) bp);
@@ -578,17 +619,17 @@ static int pop_f_opcode_init(CSOUND *csound, POP_OPCODE *p)
       p->h.opadr = (int (*)(CSOUND *, void *)) pop_f_opcode_perf;
       p->initDone = 1;
     }
-    if (p->pp->curBundle == NULL)
+    if (UNLIKELY(p->pp->curBundle == NULL))
       return csoundStack_EmptyError(p);
     bp = p->pp->curBundle;
     ofsp = (int*) ((char*) bp + (int) csoundStack_Align((int) sizeof(void*)));
     offs = p->argMap[3];
-    if (offs != *ofsp)
+    if (UNLIKELY(offs != *ofsp))
       csoundStack_TypeError(p);
     ofsp++;
-    if (*ofsp != CS_STACK_END)
+    if (UNLIKELY(*ofsp != CS_STACK_END))
       csoundStack_TypeError(p);
-    fassign(csound,
+    fsg_assign(csound,
             (PVSDAT*) p->args[0],
             *((PVSDAT**) ((char*) bp + (int) (offs & (int) 0x00FFFFFF))));
     p->pp->curBundle = *((void**) bp);
@@ -597,63 +638,22 @@ static int pop_f_opcode_init(CSOUND *csound, POP_OPCODE *p)
     return OK;
 }
 
- /* ------------------------------------------------------------------------ */
 
-typedef struct MONITOR_OPCODE_ {
-    OPDS    h;
-    MYFLT   *ar[24];
-} MONITOR_OPCODE;
-
-static int monitor_opcode_perf(CSOUND *csound, MONITOR_OPCODE *p)
-{
-    int     i, j;
-
-    if (csound->spoutactive) {
-      int   k = 0;
-      i = 0;
-      do {
-        j = 0;
-        do {
-          p->ar[j][i] = csound->spout[k++];
-        } while (++j < csound->nchnls);
-      } while (++i < csound->ksmps);
-    }
-    else {
-      j = 0;
-      do {
-        i = 0;
-        do {
-          p->ar[j][i] = FL(0.0);
-        } while (++i < csound->ksmps);
-      } while (++j < csound->nchnls);
-    }
-    return OK;
-}
-
-static int monitor_opcode_init(CSOUND *csound, MONITOR_OPCODE *p)
-{
-    if (csound->GetOutputArgCnt(p) != csound->nchnls)
-      return csound->InitError(csound, Str("number of arguments != nchnls"));
-    p->h.opadr = (SUBR) monitor_opcode_perf;
-    return OK;
-}
 
  /* ------------------------------------------------------------------------ */
 
 static OENTRY localops[] = {
-  { "stack",  sizeof(STACK_OPCODE), 1,  "",                                "i",
+  { "stack",  sizeof(STACK_OPCODE), SK|_QQ, 1,  "",   "i",
       (SUBR) stack_opcode_init, (SUBR) NULL,                      (SUBR) NULL },
-  { "push",   sizeof(PUSH_OPCODE),  3,  "",                                "N",
+  { "push",   sizeof(PUSH_OPCODE),  SK|_QQ, 3,  "",   "N",
       (SUBR) push_opcode_init,  (SUBR) notinit_opcode_stub_perf,  (SUBR) NULL },
-  { "pop",    sizeof(POP_OPCODE),   3,  "NNNNNNNNNNNNNNNNNNNNNNNNNNNNNNN", "",
+  { "pop",    sizeof(POP_OPCODE),   SK|_QQ, 3,
+                                   "NNNNNNNNNNNNNNNNNNNNNNNNNNNNNNN", "",
       (SUBR) pop_opcode_init,   (SUBR) notinit_opcode_stub_perf,  (SUBR) NULL },
-  { "push_f", sizeof(PUSH_OPCODE),  3,  "",                                "f",
+  { "push_f", sizeof(PUSH_OPCODE),  SK|_QQ, 3,  "",   "f",
       (SUBR) push_f_opcode_init, (SUBR) notinit_opcode_stub_perf, (SUBR) NULL },
-  { "pop_f",  sizeof(POP_OPCODE),   3,  "f",                               "",
-      (SUBR) pop_f_opcode_init,  (SUBR) notinit_opcode_stub_perf, (SUBR) NULL },
-  /* ----------------------------------------------------------------------- */
-  { "monitor",  sizeof(MONITOR_OPCODE), 3,  "mmmmmmmmmmmmmmmmmmmmmmmm", "",
-    (SUBR) monitor_opcode_init, (SUBR) notinit_opcode_stub_perf,  (SUBR) NULL }
+  { "pop_f",  sizeof(POP_OPCODE),   SK|_QQ, 3,  "f",   "",
+      (SUBR) pop_f_opcode_init,  (SUBR) notinit_opcode_stub_perf, (SUBR) NULL }
 };
 
 LINKAGE
