@@ -24,37 +24,55 @@
 /* vbap.c
 
 assisting functions for VBAP
-functions for loudspeaker table initialization */
+functions for loudspeaker table initialization
+Re-written to take flexible number of outputs by JPff 2012 */
 
-#include "csdl.h"
+
+#include "csoundCore.h"
+#include "interlocks.h"
 #include "vbap.h"
 #include <math.h>
 #include <stdio.h>
 #include <stdlib.h>
+#include "interlocks.h"
 
-static void choose_ls_triplets(CSOUND *csound, ls lss[CHANNELS],
-                               ls_triplet_chain **ls_triplets,
-                               int ls_amount);
-static int any_ls_inside_triplet(int, int, int, ls *, int);
+/* static void choose_ls_triplets(CSOUND *csound, ls *lss, */
+/*                                ls_triplet_chain **ls_triplets, */
+/*                                int ls_amount, int channels); */
+static int any_ls_inside_triplet(int, int, int, ls[], int);
 static void add_ldsp_triplet(CSOUND *csound, int i, int j, int k,
                              ls_triplet_chain **ls_triplets,
                              ls *lss);
 static void calculate_3x3_matrixes(CSOUND *csound,
                                    ls_triplet_chain *ls_triplets,
-                                   ls lss[CHANNELS], int ls_amount);
-static void choose_ls_tuplets(CSOUND *csound, ls lss[CHANNELS],
+                                   ls lss[], int ls_amount, int ind);
+static void choose_ls_tuplets(CSOUND *csound, ls lss[],
                               ls_triplet_chain **ls_triplets,
-                              int ls_amount);
-static void sort_2D_lss(ls lss[CHANNELS], int sorted_lss[CHANNELS],
+                              int ls_amount, int ind);
+static void sort_2D_lss(ls lss[], int sorted_lss[],
                         int ls_amount);
 
-static MYFLT *create_ls_table(CSOUND *csound, size_t cnt)
+static inline MYFLT vec_prod(CART_VEC v1, CART_VEC v2)
 {
-    csound->DestroyGlobalVariable(csound, "vbap_ls_table");
-    if (csound->CreateGlobalVariable(csound, "vbap_ls_table",
-                                             cnt * sizeof(MYFLT)) != 0)
-      csound->Die(csound, Str("vbap: error allocating loudspeaker table"));
-    return get_ls_table(csound);
+    return (v1.x*v2.x + v1.y*v2.y + v1.z*v2.z);
+}
+
+static inline MYFLT vec_length(CART_VEC v1)
+{
+    return SQRT(v1.x*v1.x + v1.y*v1.y + v1.z*v1.z);
+}
+
+static MYFLT *create_ls_table(CSOUND *csound, size_t cnt, int ind)
+{
+    char name[24];
+    snprintf(name, 24, "vbap_ls_table_%d", ind);
+    csound->DestroyGlobalVariable(csound, name);
+    if (UNLIKELY(csound->CreateGlobalVariable(csound, name,
+                                              cnt * sizeof(MYFLT)) != 0)) {
+      csound->ErrorMsg(csound, Str("vbap: error allocating loudspeaker table"));
+      return NULL;
+    }
+    return (MYFLT*) (csound->QueryGlobalVariableNoCheck(csound, name));
 }
 
 void calc_vbap_gns(int ls_set_am, int dim, LS_SET *sets,
@@ -120,19 +138,16 @@ void calc_vbap_gns(int ls_set_am, int dim, LS_SET *sets,
       sets[j].set_gains[2] = FL(1.0);
       }
 
-/*     for (i=0;i<ls_amount;i++) { */
-/*       gains[i]=FL(0.0); */
-/*     } */
     memset(gains, 0, ls_amount*sizeof(MYFLT));
 
     gains[sets[j].ls_nos[0]-1] = sets[j].set_gains[0];
     gains[sets[j].ls_nos[1]-1] = sets[j].set_gains[1];
-    if(dim==3) gains[sets[j].ls_nos[2]-1] = sets[j].set_gains[2];
+    if (dim==3) gains[sets[j].ls_nos[2]-1] = sets[j].set_gains[2];
 
-      for (i=0;i<ls_amount;i++) {
+    for (i=0;i<ls_amount;i++) {
       if (gains[i]<LOWEST_ACCEPTABLE_WT)
         gains[i]=FL(0.0);
-        }
+    }
 }
 
 void scale_angles(ANG_VEC *avec)
@@ -187,28 +202,27 @@ void cart_to_angle(CART_VEC cvec, ANG_VEC *avec)
     MYFLT tmp, tmp2, tmp3, tmp4;
     MYFLT atorad = (TWOPI_F / FL(360.0));
 
-    tmp3 = (MYFLT) sqrt(1.0- (double)cvec.z*cvec.z);
-    if (fabs(tmp3) > 0.001) {
+    tmp3 = SQRT(FL(1.0) - cvec.z*cvec.z);
+    if (FABS(tmp3) > FL(0.001)) {
       tmp4 = (cvec.x / tmp3);
       if (tmp4 > FL(1.0)) tmp4 = FL(1.0);
       if (tmp4 < -FL(1.0)) tmp4 = -FL(1.0);
-      tmp = (MYFLT) acos((double) tmp4 );
+      tmp = ACOS(tmp4 );
     }
     else {
       tmp = FL(10000.0);
     }
-    if (fabs(cvec.y) <= 0.001)
+    if (FABS(cvec.y) <= FL(0.001))
       tmp2 = FL(1.0);
     else
-      tmp2 = (MYFLT)(cvec.y / fabs(cvec.y));
+      tmp2 = cvec.y / FABS(cvec.y);
     tmp *= tmp2;
-    if (fabs(tmp) <= PI) {
+    if (FABS(tmp) <= PI_F) {
       avec->azi =  tmp;
       avec->azi /= atorad;
     }
-    avec->ele = (MYFLT) asin((double) cvec.z);
-    avec->length = (MYFLT)sqrt((double)(cvec.x * cvec.x + cvec.y *
-                                        cvec.y + cvec.z * cvec.z));
+    avec->ele = ASIN(cvec.z);
+    avec->length = SQRT(cvec.x * cvec.x + cvec.y * cvec.y + cvec.z * cvec.z);
     avec->ele /= atorad;
 }
 
@@ -216,14 +230,12 @@ void angle_to_cart_II(ANG_VEC *from, CART_VEC *to)
      /* conversion, double*/
 {
     MYFLT ang2rad = TWOPI_F / FL(360.0);
-    to->x= (MYFLT) (cos((double)(from->azi * ang2rad)) *
-                    cos((double) (from->ele * ang2rad)));
-    to->y= (MYFLT) (sin((double)(from->azi * ang2rad)) *
-                    cos((double) (from->ele * ang2rad)));
-    to->z= (MYFLT) (sin((double) (from->ele * ang2rad)));
+    to->x= COS(from->azi * ang2rad) * COS(from->ele * ang2rad);
+    to->y= SIN(from->azi * ang2rad) * COS(from->ele * ang2rad);
+    to->z= SIN(from->ele * ang2rad);
 }
 
-MYFLT vol_p_side_lgth(int i, int j,int k, ls  lss[CHANNELS] )
+MYFLT vol_p_side_lgth(int i, int j,int k, ls  lss[] )
 {
   /* calculate volume of the parallelepiped defined by the loudspeaker
      direction vectors and divide it with total length of the triangle sides.
@@ -232,17 +244,17 @@ MYFLT vol_p_side_lgth(int i, int j,int k, ls  lss[CHANNELS] )
     MYFLT volper, lgth;
     CART_VEC xprod;
     cross_prod(lss[i].coords, lss[j].coords, &xprod);
-    volper = (MYFLT)fabs(vec_prod(xprod, lss[k].coords));
-    lgth = (MYFLT)(fabs(vec_angle(lss[i].coords,lss[j].coords))
-            + fabs(vec_angle(lss[i].coords,lss[k].coords))
-            + fabs(vec_angle(lss[j].coords,lss[k].coords)));
-    if (lgth>0.00001)
+    volper = FABS(vec_prod(xprod, lss[k].coords));
+    lgth =    FABS(vec_angle(lss[i].coords,lss[j].coords))
+            + FABS(vec_angle(lss[i].coords,lss[k].coords))
+            + FABS(vec_angle(lss[j].coords,lss[k].coords));
+    if (lgth>FL(0.00001))
       return volper / lgth;
     else
       return FL(0.0);
 }
 
-static void choose_ls_triplets(CSOUND *csound, ls lss[CHANNELS],
+static void choose_ls_triplets(CSOUND *csound, ls *lss,
                                struct ls_triplet_chain **ls_triplets,
                                int ls_amount)
   /* Selects the loudspeaker triplets, and
@@ -262,17 +274,18 @@ static void choose_ls_triplets(CSOUND *csound, ls lss[CHANNELS],
     MYFLT distance;
     struct ls_triplet_chain *trip_ptr, *prev, *tmp_ptr;
 
-    if (ls_amount == 0) {
-      csound->Die(csound, Str("Number of loudspeakers is zero\nExiting"));
+    if (UNLIKELY(ls_amount == 0)) {
+      csound->ErrorMsg(csound, Str("Number of loudspeakers is zero\nExiting"));
+      return;
     }
 
-    connections = csound->Calloc(csound, CHANNELS * CHANNELS * sizeof(int));
-    distance_table = csound->Calloc(csound, ((CHANNELS * (CHANNELS - 1)) / 2)
-                                            * sizeof(MYFLT));
-    distance_table_i = csound->Calloc(csound, ((CHANNELS * (CHANNELS - 1)) / 2)
-                                              * sizeof(int));
-    distance_table_j = csound->Calloc(csound, ((CHANNELS * (CHANNELS - 1)) / 2)
-                                              * sizeof(int));
+    connections = csound->Calloc(csound, ls_amount * ls_amount * sizeof(int));
+    distance_table =
+      csound->Calloc(csound, ((ls_amount * (ls_amount - 1)) / 2)* sizeof(MYFLT));
+    distance_table_i =
+      csound->Calloc(csound, ((ls_amount * (ls_amount - 1)) / 2)* sizeof(int));
+    distance_table_j =
+      csound->Calloc(csound, ((ls_amount * (ls_amount - 1)) / 2)* sizeof(int));
 
 /*  i_ptr = (int *) connections; */
 /*  for (i=0;i< ((CHANNELS) * (CHANNELS )); i++) */
@@ -282,12 +295,12 @@ static void choose_ls_triplets(CSOUND *csound, ls lss[CHANNELS],
     for (j=i+1;j<ls_amount;j++)
       for (k=j+1;k<ls_amount;k++) {
         if (vol_p_side_lgth(i,j, k, lss) > MIN_VOL_P_SIDE_LGTH) {
-          connections[i+CHANNELS*j]=1;
-          connections[j+CHANNELS*i]=1;
-          connections[i+CHANNELS*k]=1;
-          connections[k+CHANNELS*i]=1;
-          connections[j+CHANNELS*k]=1;
-          connections[k+CHANNELS*j]=1;
+          connections[i+ls_amount*j]=1;
+          connections[j+ls_amount*i]=1;
+          connections[i+ls_amount*k]=1;
+          connections[k+ls_amount*i]=1;
+          connections[j+ls_amount*k]=1;
+          connections[k+ls_amount*j]=1;
           add_ldsp_triplet(csound, i, j, k, ls_triplets, lss);
         }
       }
@@ -299,8 +312,8 @@ static void choose_ls_triplets(CSOUND *csound, ls lss[CHANNELS],
 
     for (i=0;i<ls_amount;i++) {
       for (j=(i+1);j<ls_amount; j++) {
-        if (connections[i+CHANNELS*j] == 1) {
-          distance = (MYFLT)fabs(vec_angle(lss[i].coords,lss[j].coords));
+        if (connections[i+ls_amount*j] == 1) {
+          distance = FABS(vec_angle(lss[i].coords,lss[j].coords));
           k=0;
 
           while (distance_table[k] < distance)
@@ -325,13 +338,13 @@ static void choose_ls_triplets(CSOUND *csound, ls lss[CHANNELS],
     for (i=0; i<(table_size); i++) {
       int fst_ls = distance_table_i[i];
       int sec_ls = distance_table_j[i];
-      if (connections[fst_ls+CHANNELS*sec_ls] == 1)
+      if (connections[fst_ls+ls_amount*sec_ls] == 1)
         for (j=0; j<ls_amount; j++)
           for (k=j+1; k<ls_amount; k++)
             if ( (j!=fst_ls) && (k != sec_ls) && (k!=fst_ls) && (j != sec_ls))
               if (lines_intersect(fst_ls, sec_ls, j,k,lss) == 1) {
-                connections[j+CHANNELS*k] = 0;
-                connections[k+CHANNELS*j] = 0;
+                connections[j+ls_amount*k] = 0;
+                connections[k+ls_amount*j] = 0;
               }
     }
 
@@ -344,9 +357,9 @@ static void choose_ls_triplets(CSOUND *csound, ls lss[CHANNELS],
       i = trip_ptr->ls_nos[0];
       j = trip_ptr->ls_nos[1];
       k = trip_ptr->ls_nos[2];
-      if (connections[i+CHANNELS*j] == 0 ||
-          connections[i+CHANNELS*k] == 0 ||
-          connections[j+CHANNELS*k] == 0 ||
+      if (connections[i+ls_amount*j] == 0 ||
+          connections[i+ls_amount*k] == 0 ||
+          connections[j+ls_amount*k] == 0 ||
           any_ls_inside_triplet(i,j,k,lss,ls_amount) == 1) {
         if (prev != NULL) {
           prev->next = trip_ptr->next;
@@ -366,15 +379,15 @@ static void choose_ls_triplets(CSOUND *csound, ls lss[CHANNELS],
         trip_ptr = trip_ptr->next;
       }
     }
-    csound->Free(csound, connections);
-    csound->Free(csound, distance_table);
-    csound->Free(csound, distance_table_i);
-    csound->Free(csound, distance_table_j);
+    csound->Free(csound,connections);
+    csound->Free(csound,distance_table);
+    csound->Free(csound,distance_table_i);
+    csound->Free(csound,distance_table_j);
 }
 
 /* returns 1 if there is loudspeaker(s) inside given ls triplet */
 
-static int any_ls_inside_triplet(int a, int b, int c, ls lss[CHANNELS],
+static int any_ls_inside_triplet(int a, int b, int c, ls lss[],
                                  int ls_amount)
 {
     MYFLT invdet;
@@ -411,7 +424,7 @@ static int any_ls_inside_triplet(int a, int b, int c, ls lss[CHANNELS],
           tmp = lss[i].coords.x * invmx[0 + j*3];
           tmp += lss[i].coords.y * invmx[1 + j*3];
           tmp += lss[i].coords.z * invmx[2 + j*3];
-          if (tmp < -0.001)
+          if (tmp < -FL(0.001))
             this_inside = 0;
         }
         if (this_inside == 1)
@@ -423,7 +436,7 @@ static int any_ls_inside_triplet(int a, int b, int c, ls lss[CHANNELS],
 
 static void add_ldsp_triplet(CSOUND *csound, int i, int j, int k,
                              struct ls_triplet_chain **ls_triplets,
-                             ls lss[CHANNELS])
+                             ls lss[])
 {
     struct ls_triplet_chain *ls_ptr, *prev;
     ls_ptr = *ls_triplets;
@@ -452,10 +465,10 @@ MYFLT angle_in_base(CART_VEC vb1,CART_VEC vb2,CART_VEC vec)
 {
     MYFLT tmp1,tmp2;
     tmp1 = vec_prod(vec,vb2);
-    if (fabs((double)tmp1) <= 0.001)
+    if (FABS(tmp1) <= FL(0.001))
       tmp2 = FL(1.0);
     else
-      tmp2 = tmp1 / (MYFLT)fabs((double)tmp1);
+      tmp2 = tmp1 / FABS(tmp1);
     return (vec_angle(vb1,vec) * tmp2);
 }
 
@@ -467,7 +480,7 @@ MYFLT vec_angle(CART_VEC v1, CART_VEC v2)
       inner= FL(1.0);
     if (inner < -FL(1.0))
       inner = -FL(1.0);
-    return (MYFLT) acos((double) inner);
+    return ACOS(inner);
 }
 
 void vec_mean(CART_VEC v1, CART_VEC v2, CART_VEC *v3)
@@ -477,16 +490,6 @@ void vec_mean(CART_VEC v1, CART_VEC v2, CART_VEC *v3)
     v3->z=(v1.z+v2.z)*FL(0.5);
 }
 
-MYFLT vec_length(CART_VEC v1)
-{
-    return ((MYFLT)sqrt((double)(v1.x*v1.x + v1.y*v1.y + v1.z*v1.z)));
-}
-
-MYFLT vec_prod(CART_VEC v1, CART_VEC v2)
-{
-    return (v1.x*v2.x + v1.y*v2.y + v1.z*v2.z);
-}
-
 void cross_prod(CART_VEC v1,CART_VEC v2,
                 CART_VEC *res)
 {
@@ -507,7 +510,7 @@ void vec_print(CSOUND *csound, CART_VEC v)
 
 }
 
-int lines_intersect(int i,int j,int k,int l,ls  lss[CHANNELS])
+int lines_intersect(int i,int j,int k,int l,ls  lss[])
   /* checks if two lines intersect on 3D sphere
      see theory in paper Pulkki, V. Lokki, T. "Creating Auditory Displays
      with Multiple Loudspeakers Using VBAP: A Case Study with
@@ -541,16 +544,16 @@ int lines_intersect(int i,int j,int k,int l,ls  lss[CHANNELS])
     dist_lnv3 = (vec_angle(neg_v3,lss[l].coords));
 
     /* if one of loudspeakers is close to crossing point, don't do anything*/
-    if (fabs(dist_iv3) <= 0.01 || fabs(dist_jv3) <= 0.01 ||
-        fabs(dist_kv3) <= 0.01 || fabs(dist_lv3) <= 0.01 ||
-        fabs(dist_inv3) <= 0.01 || fabs(dist_jnv3) <= 0.01 ||
-        fabs(dist_knv3) <= 0.01 || fabs(dist_lnv3) <= 0.01 )
+    if (FABS(dist_iv3) <= FL(0.01) || FABS(dist_jv3) <= FL(0.01) ||
+        FABS(dist_kv3) <= FL(0.01) || FABS(dist_lv3) <= FL(0.01) ||
+        FABS(dist_inv3) <= FL(0.01) || FABS(dist_jnv3) <= FL(0.01) ||
+        FABS(dist_knv3) <= FL(0.01) || FABS(dist_lnv3) <= FL(0.01) )
       return(0);
 
-    if (((fabs(dist_ij - (dist_iv3 + dist_jv3)) <= 0.01 ) &&
-         (fabs(dist_kl - (dist_kv3 + dist_lv3))  <= 0.01)) ||
-        ((fabs(dist_ij - (dist_inv3 + dist_jnv3)) <= 0.01)  &&
-         (fabs(dist_kl - (dist_knv3 + dist_lnv3)) <= 0.01 ))) {
+    if (((FABS(dist_ij - (dist_iv3 + dist_jv3)) <= FL(0.01) ) &&
+         (FABS(dist_kl - (dist_kv3 + dist_lv3))  <= FL(0.01))) ||
+        ((FABS(dist_ij - (dist_inv3 + dist_jnv3)) <= FL(0.01))  &&
+         (FABS(dist_kl - (dist_knv3 + dist_lnv3)) <= FL(0.01) ))) {
       return (1);
     }
     else {
@@ -558,33 +561,33 @@ int lines_intersect(int i,int j,int k,int l,ls  lss[CHANNELS])
     }
 }
 
-int vbap_ls_init (CSOUND *csound, VBAP_LS_INIT *p)
+static inline int vbap_ls_init_sr (CSOUND *csound, int dim, int count,
+                            MYFLT **f, int layout)
      /* Inits the loudspeaker data. Calls choose_ls_tuplets or _triplets
         according to current dimension. The inversion matrices are
         stored in transposed form to ease calculation at run time.*/
 {
-    int dim, count;
     struct ls_triplet_chain *ls_triplets = NULL;
-    ls lss[CHANNELS];
+    ls *lss = malloc(sizeof(ls)*count);
 
     ANG_VEC a_vector;
     CART_VEC c_vector;
     int i=0,j;
-    int ls_amount;
 
-    dim = (int) *p->dim;
-    csound->Message(csound, "dim : %d\n",dim);
-    if (!((dim==2) || (dim == 3))) {
-      csound->Die(csound, Str("Error in loudspeaker dimension."));
+    //dim = (int) *p->dim;
+    csound->Warning(csound, "dim : %d\n",dim);
+    if (UNLIKELY(!((dim==2) || (dim == 3)))) {
+      csound->ErrorMsg(csound, Str("Error in loudspeaker dimension."));
+      return NOTOK;
     }
-    count = (int) *p->ls_amount;
+    //count = (int) *p->ls_amount;
     for (j=1;j<=count;j++) {
       if (dim == 3) {
-        a_vector.azi= (MYFLT) *p->f[2*j-2];
-        a_vector.ele= (MYFLT) *p->f[2*j-1];
+        a_vector.azi= (MYFLT) *f[2*j-2];
+        a_vector.ele= (MYFLT) *f[2*j-1];
       }
       else if (dim == 2) {
-        a_vector.azi= (MYFLT) *p->f[j-1];
+        a_vector.azi= (MYFLT) *f[j-1];
         a_vector.ele=FL(0.0);
       }
       angle_to_cart_II(&a_vector,&c_vector);
@@ -596,24 +599,52 @@ int vbap_ls_init (CSOUND *csound, VBAP_LS_INIT *p)
       lss[i].angles.length = FL(1.0);
       i++;
     }
-    ls_amount = (int)*p->ls_amount;
-    if (ls_amount < dim) {
-      csound->Die(csound, Str("Too few loudspeakers"));
+    //ls_amount = (int)*p->ls_amount;
+    if (UNLIKELY(count < dim)) {
+      free(lss);
+      csound->ErrorMsg(csound, Str("Too few loudspeakers"));
+      return NOTOK;
     }
 
     if (dim == 3) {
-      choose_ls_triplets(csound, lss, &ls_triplets,ls_amount);
-      calculate_3x3_matrixes(csound, ls_triplets,lss,ls_amount);
+      choose_ls_triplets(csound, lss, &ls_triplets,count);
+      calculate_3x3_matrixes(csound, ls_triplets,lss,count, layout);
     }
     else if (dim ==2) {
-      choose_ls_tuplets(csound, lss, &ls_triplets,ls_amount);
+      choose_ls_tuplets(csound, lss, &ls_triplets,count, layout);
     }
+    free(lss);
     return OK;
 }
 
+int vbap_ls_init (CSOUND *csound, VBAP_LS_INIT *p)
+{
+    int dim = (int) *p->dim;
+    MYFLT  layout = (*p->dim-dim)*100;
+    return vbap_ls_init_sr(csound, dim, (int) *p->ls_amount, p->f, round(layout));
+}
+
+int vbap_ls_inita (CSOUND *csound, VBAP_LS_INITA *p)
+{
+    int dim = (int) *p->dim;
+    MYFLT  layout = (*p->dim-dim)*100;
+    int i, n = (int)*p->ls_amount;
+    /* if (n>CHANNELS) */
+    /*   return csound->InitError(csound, Str("Too many speakers (%n)\n"), n); */
+    if (n>p->a->sizes[0])
+      return csound->InitError(csound, Str("Too little data speakers (%n)\n"),
+                              n>p->a->sizes[0]);
+    MYFLT  **f = csound->Malloc(csound, 2*sizeof(MYFLT*)*n);
+    // Transfer values to pointers
+    for (i=0; i<2*n; i++) f[i] = &(p->a->data[i]);
+    n = vbap_ls_init_sr(csound, dim, n, f, round(layout));
+    csound->Free(csound, f);
+    return n;
+}
+
 static void calculate_3x3_matrixes(CSOUND *csound,
                                    struct ls_triplet_chain *ls_triplets,
-                                   ls lss[CHANNELS], int ls_amount)
+                                   ls lss[], int ls_amount, int ind)
      /* Calculates the inverse matrices for 3D */
 {
     MYFLT invdet;
@@ -623,8 +654,9 @@ static void calculate_3x3_matrixes(CSOUND *csound,
     struct ls_triplet_chain *tr_ptr = ls_triplets;
     int triplet_amount = 0, i,j,k;
 
-    if (tr_ptr == NULL) {
-      csound->Die(csound, Str("Not valid 3-D configuration"));
+    if (UNLIKELY(tr_ptr == NULL)) {
+      csound->ErrorMsg(csound, Str("Not valid 3-D configuration"));
+      return;
     }
 
     /* counting triplet amount */
@@ -634,7 +666,7 @@ static void calculate_3x3_matrixes(CSOUND *csound,
     }
 
     /* calculations and data storage to a global array */
-    ls_table = create_ls_table(csound, triplet_amount * 12 + 3);
+    ls_table = create_ls_table(csound, triplet_amount * 12 + 3, ind);
     ls_table[0] = FL(3.0);  /* dimension */
     ls_table[1] = (MYFLT) ls_amount;
     ls_table[2] = (MYFLT) triplet_amount;
@@ -670,40 +702,40 @@ static void calculate_3x3_matrixes(CSOUND *csound,
     }
 
     k = 3;
-    csound->Message(csound, Str("\nConfigured loudspeakers\n"));
+    csound->Warning(csound, Str("\nConfigured loudspeakers\n"));
     for (i = 0; i < triplet_amount; i++) {
-      csound->Message(csound, Str("Triplet %d Loudspeakers: "), i);
+      csound->Warning(csound, Str("Triplet %d Loudspeakers: "), i);
       for (j = 0; j < 3; j++) {
-        csound->Message(csound, "%d ", (int) ls_table[k++]);
+        csound->Warning(csound, "%d ", (int) ls_table[k++]);
       }
-      csound->Message(csound, "\n");
+      csound->Warning(csound, "\n");
 
-   /* printf("\nMatrix "); */
-      for (j = 0; j < 9; j++) {
-     /* printf("%f ", ls_table[k]); */
-        k++;
-      }
-   /* printf("\n\n"); */
+    /* printf("\nMatrix ");  */
+    /*   for (j = 0; j < 9; j++) { */
+    /*     printf("%f ", ls_table[k]);  */
+    /*     k++; */
+    /*   } */
+    /* printf("\n\n"); */
     }
 }
 
 static void choose_ls_tuplets(CSOUND *csound,
-                              ls lss[CHANNELS],
+                              ls lss[],
                               ls_triplet_chain **ls_triplets,
-                              int ls_amount)
+                              int ls_amount, int ind)
      /* selects the loudspeaker pairs, calculates the inversion
         matrices and stores the data to a global array */
 {
     int i, j, k;
-    int sorted_lss[CHANNELS];
-    int exist[CHANNELS];
+    int *sorted_lss = (int*)malloc(sizeof(int)*ls_amount);
+    int *exist = (int*)calloc(1,sizeof(int)*ls_amount);
     int amount = 0;
-    MYFLT inv_mat[CHANNELS][4], *ls_table, *ptr;
-    int ftable_size;
+    MYFLT *inv_mat = (MYFLT*)malloc(4*sizeof(MYFLT)*ls_amount), *ls_table, *ptr;
+    //int ftable_size;
 
-    for (i=0;i<CHANNELS;i++) {
-      exist[i]=0;
-    }
+    /* for (i=0;i<CHANNELS;i++) { */
+    /*   exist[i]=0; */
+    /* } */
 
     /* sort loudspeakers according their aximuth angle */
     sort_2D_lss(lss,sorted_lss,ls_amount);
@@ -713,41 +745,51 @@ static void choose_ls_tuplets(CSOUND *csound,
       /*csound->Message(csound, "%d %d %f %f\n",sorted_lss[i],sorted_lss[i+1],
         lss[sorted_lss[i]].angles.azi,
         lss[sorted_lss[i+1]].angles.azi);*/
-      if ((lss[sorted_lss[i+1]].angles.azi -
-           lss[sorted_lss[i]].angles.azi) <= (PI - 0.175)) {
-        if (calc_2D_inv_tmatrix( lss[sorted_lss[i]].angles.azi,
-                                 lss[sorted_lss[i+1]].angles.azi,
-                                 inv_mat[i]) != 0) {
+      if (LIKELY((lss[sorted_lss[i+1]].angles.azi -
+                  lss[sorted_lss[i]].angles.azi) <= (PI - 0.0175))) {
+        if (LIKELY(calc_2D_inv_tmatrix( lss[sorted_lss[i]].angles.azi,
+                                        lss[sorted_lss[i+1]].angles.azi,
+                                        &inv_mat[4*i]) != 0)) {
           exist[i]=1;
           amount++;
         }
       }
+      else  csound->Warning(csound, Str("Pair of speakers at %f and %f ignored\n"),
+                            lss[sorted_lss[i]].angles.azi*FL(180.0)/PI_F,
+                            lss[sorted_lss[i+1]].angles.azi*FL(180.0)/PI_F);
     }
 
-    if (((6.283 - lss[sorted_lss[ls_amount-1]].angles.azi)
-         +lss[sorted_lss[0]].angles.azi) <= (PI - 0.175)) {
-      if (calc_2D_inv_tmatrix(lss[sorted_lss[ls_amount-1]].angles.azi,
-                              lss[sorted_lss[0]].angles.azi,
-                              inv_mat[ls_amount-1]) != 0) {
+    if (LIKELY(((TWOPI_F - lss[sorted_lss[ls_amount-1]].angles.azi)
+                +lss[sorted_lss[0]].angles.azi) < (PI - 0.0175))) {
+      //printf("less than PI type 2- 0.175\n");
+      if (LIKELY(calc_2D_inv_tmatrix(lss[sorted_lss[ls_amount-1]].angles.azi,
+                                     lss[sorted_lss[0]].angles.azi,
+                                     &inv_mat[4*(ls_amount-1)]) != 0)) {
         exist[ls_amount-1]=1;
         amount++;
       }
     }
+    else  csound->Warning(csound, Str("Pair of speakers at %f and %f ignored\n"),
+                          lss[sorted_lss[ls_amount-1]].angles.azi*FL(180.0)/PI_F,
+                          lss[sorted_lss[0]].angles.azi*FL(180.0)/PI_F);
 
+    if (UNLIKELY(amount==0))
+      csound->InitError(csound, Str("insufficient valid speakers"));
+
+#if 0
     if ( amount*6 + 6 <= 16) ftable_size = 16;
     else if ( amount*6 + 6 <= 32) ftable_size = 32;
     else if ( amount*6 + 6 <= 64) ftable_size = 64;
     else if ( amount*6 + 6 <= 128) ftable_size = 128;
     else if ( amount*6 + 6 <= 256) ftable_size = 256;
     else if ( amount*6 + 6 <= 1024) ftable_size = 1024;
-#if 0
     csound->Message(csound,
                     "Loudspeaker matrices calculated with configuration : ");
     for (i=0; i< ls_amount; i++)
       csound->Message(csound, "%.1f ", lss[i].angles.azi / atorad);
     csound->Message(csound, "\n");
 #endif
-    ls_table = create_ls_table(csound, amount * 6 + 3 + 100);
+    ls_table = create_ls_table(csound, amount * 6 + 3 + 100, ind);
     ls_table[0] = FL(2.0);  /* dimension */
     ls_table[1] = (MYFLT) ls_amount;
     ls_table[2] = (MYFLT) amount;
@@ -757,7 +799,7 @@ static void choose_ls_tuplets(CSOUND *csound,
         *(ptr++) = (MYFLT)sorted_lss[i]+1;
         *(ptr++) = (MYFLT)sorted_lss[i+1]+1;
         for (j=0;j<4;j++) {
-          *(ptr++) = inv_mat[i][j];
+          *(ptr++) = inv_mat[i*ls_amount+j];
         }
       }
     }
@@ -765,28 +807,29 @@ static void choose_ls_tuplets(CSOUND *csound,
       *(ptr++) = (MYFLT)sorted_lss[ls_amount-1]+1;
       *(ptr++) = (MYFLT)sorted_lss[0]+1;
       for (j=0;j<4;j++) {
-        *(ptr++) = inv_mat[ls_amount-1][j];
+        *(ptr++) = inv_mat[ls_amount*(ls_amount-1)+j];
       }
     }
     k=3;
-    csound->Message(csound, Str("\nConfigured loudspeakers\n"));
+    csound->Warning(csound, Str("\nConfigured loudspeakers\n"));
     for (i=0; i < amount; i++) {
-      csound->Message(csound, Str("Pair %d Loudspeakers: "), i);
+      csound->Warning(csound, Str("Pair %d Loudspeakers: "), i);
       for (j=0; j < 2; j++) {
-        csound->Message(csound, "%d ", (int) ls_table[k++]);
+        csound->Warning(csound, "%d ", (int) ls_table[k++]);
       }
-      csound->Message(csound, "\n");
+      csound->Warning(csound, "\n");
 
-   /* csound->Message(csound, "\nMatrix "); */
+    csound->Message(csound, "\nMatrix ");
       for (j=0; j < 4; j++) {
-     /* csound->Message(csound, "%f ", ls_table[k]); */
+      csound->Message(csound, "%f ", ls_table[k]);
         k++;
       }
-   /* csound->Message(csound, "\n\n"); */
+      csound->Message(csound, "\n\n");
     }
+    free(sorted_lss); free(exist); free(inv_mat);
 }
 
-static void sort_2D_lss(ls lss[CHANNELS], int sorted_lss[CHANNELS],
+static void sort_2D_lss(ls lss[], int sorted_lss[],
                         int ls_amount)
 {
     int i,j,index=-1;
@@ -795,11 +838,11 @@ static void sort_2D_lss(ls lss[CHANNELS], int sorted_lss[CHANNELS],
     /* Transforming angles between -180 and 180 */
     for (i=0; i<ls_amount; i++) {
       angle_to_cart_II(&lss[i].angles, &lss[i].coords);
-      lss[i].angles.azi = (MYFLT) acos((double) lss[i].coords.x);
-      if (fabs((double)lss[i].coords.y) <= 0.001)
+      lss[i].angles.azi = ACOS(lss[i].coords.x);
+      if (FABS(lss[i].coords.y) <= FL(0.001))
         tmp = FL(1.0);
       else
-        tmp = lss[i].coords.y / (MYFLT)fabs((double)lss[i].coords.y);
+        tmp = lss[i].coords.y / FABS(lss[i].coords.y);
       lss[i].angles.azi *= tmp;
       /*printf("tulos %f",    lss[i].angles.azi);*/
     }
@@ -825,12 +868,12 @@ int calc_2D_inv_tmatrix(MYFLT azi1,MYFLT azi2, MYFLT inv_mat[4])
 {
     MYFLT x1,x2,x3,x4; /* x1 x3 */
     MYFLT det;
-    x1 = (MYFLT) cos((double) azi1 );
-    x2 = (MYFLT) sin((double) azi1 );
-    x3 = (MYFLT) cos((double) azi2 );
-    x4 = (MYFLT) sin((double) azi2 );
+    x1 = COS(azi1 );
+    x2 = SIN(azi1 );
+    x3 = COS(azi2 );
+    x4 = SIN(azi2 );
     det = (x1 * x4) - ( x3 * x2 );
-    if (fabs((double)det) <= 0.001) {
+    if (FABS(det) <= FL(0.001)) {
       /*printf("unusable pair, det %f\n",det);*/
       inv_mat[0] = FL(0.0);
       inv_mat[1] = FL(0.0);
@@ -839,10 +882,10 @@ int calc_2D_inv_tmatrix(MYFLT azi1,MYFLT azi2, MYFLT inv_mat[4])
       return 0;
     }
     else {
-      inv_mat[0] =  (MYFLT) (x4 / det);
-      inv_mat[1] =  (MYFLT) (-x3 / det);
-      inv_mat[2] =  (MYFLT) (-x2 / det);
-      inv_mat[3] =  (MYFLT)  (x1 / det);
+      inv_mat[0] =  (x4 / det);
+      inv_mat[1] =  (-x3 / det);
+      inv_mat[2] =  (-x2 / det);
+      inv_mat[3] =  (x1 / det);
       return 1;
     }
 }
@@ -854,29 +897,29 @@ void new_spread_dir(CART_VEC *spreaddir, CART_VEC vscartdir,
     MYFLT a,b;
     MYFLT power;
     ANG_VEC tmp;
-    gamma = (MYFLT) acos((double)(vscartdir.x * spread_base.x +
-                                  vscartdir.y * spread_base.y +
-                                  vscartdir.z * spread_base.z))/PI_F*FL(180.0);
-    if (fabs(gamma) < 1) {
+    gamma = ACOS(vscartdir.x * spread_base.x +
+                 vscartdir.y * spread_base.y +
+                 vscartdir.z * spread_base.z)/PI_F*FL(180.0);
+    if (FABS(gamma) < FL(1.0)) {
       tmp.azi=azi+FL(90.0);
       tmp.ele=FL(0.0); tmp.length=FL(1.0);
       angle_to_cart(tmp, &spread_base);
-      gamma = (MYFLT) acos((double)(vscartdir.x * spread_base.x +
-                                    vscartdir.y * spread_base.y +
-                                    vscartdir.z * spread_base.z))/PI_F*FL(180.0);
+      gamma = ACOS(vscartdir.x * spread_base.x +
+                   vscartdir.y * spread_base.y +
+                   vscartdir.z * spread_base.z)/PI_F*FL(180.0);
     }
-    beta = 180 - gamma;
-    b=(MYFLT)sin((double)spread * PI / 180.0) /
-      (MYFLT)sin((double)beta * PI / 180.0);
-    a=(MYFLT)sin((180.0- (double)spread - (double)beta) * PI / FL(180.0)) /
-      (MYFLT)sin ((double)beta * PI / 180.0);
+    beta = FL(180.0) - gamma;
+    b=SIN(spread * PI_F / FL(180.0)) /
+      SIN(beta * PI_F / FL(180.0));
+    a=SIN((FL(180.0)- spread - beta) * PI_F / FL(180.0)) /
+      SIN (beta * PI_F / FL(180.0));
     spreaddir->x = a * vscartdir.x + b * spread_base.x;
     spreaddir->y = a * vscartdir.y + b * spread_base.y;
     spreaddir->z = a * vscartdir.z + b * spread_base.z;
 
-    power=(MYFLT)sqrt((double)(spreaddir->x*spreaddir->x +
-                               spreaddir->y*spreaddir->y +
-                               spreaddir->z*spreaddir->z));
+    power=SQRT(spreaddir->x*spreaddir->x +
+               spreaddir->y*spreaddir->y +
+               spreaddir->z*spreaddir->z);
     spreaddir->x /= power;
     spreaddir->y /= power;
     spreaddir->z /= power;
@@ -888,13 +931,13 @@ void new_spread_base(CART_VEC spreaddir, CART_VEC vscartdir,
     MYFLT d;
     MYFLT power;
 
-    d = (MYFLT)cos((double)spread/180.0*PI);
+    d = COS(spread/FL(180.0)*PI_F);
     spread_base->x = spreaddir.x - d * vscartdir.x;
     spread_base->y = spreaddir.y - d * vscartdir.y;
     spread_base->z = spreaddir.z - d * vscartdir.z;
-    power=(MYFLT)sqrt((double)(spread_base->x*spread_base->x +
-                               spread_base->y*spread_base->y +
-                               spread_base->z*spread_base->z));
+    power=SQRT(spread_base->x*spread_base->x +
+               spread_base->y*spread_base->y +
+               spread_base->z*spread_base->z);
     spread_base->x /= power;
     spread_base->y /= power;
     spread_base->z /= power;
@@ -902,36 +945,65 @@ void new_spread_base(CART_VEC spreaddir, CART_VEC vscartdir,
 
 #define S(x)    sizeof(x)
 
-static OENTRY localops[] = {
-  { "vbap4",      S(VBAP_FOUR),             5,  "aaaa",             "aioo",
-    (SUBR) vbap_FOUR_init,          (SUBR) NULL,    (SUBR) vbap_FOUR        },
-  { "vbap8",      S(VBAP_EIGHT),            5,  "aaaaaaaa",         "aioo",
-    (SUBR) vbap_EIGHT_init,         (SUBR) NULL,    (SUBR) vbap_EIGHT       },
-  { "vbap16",     S(VBAP_SIXTEEN),          5,  "aaaaaaaaaaaaaaaa", "aioo",
-    (SUBR) vbap_SIXTEEN_init,       (SUBR) NULL,    (SUBR) vbap_SIXTEEN     },
-  { "vbapz",      S(VBAP_ZAK),              5,  "",                 "iiaioo",
+/* static */
+static OENTRY vbap_localops[] = {
+  { "vbap.a",      S(VBAP),
+    TR, 5,  "mmmmmmmmmmmmmmmmmmmmmmmmmmmmmmmmmmmmmmmmmmmmmmmmmmmmmmmmmmmmmmmm"
+    "mmmmmmmmmmmmmmmmmmmmmmmmmmmmmmmmmmmmmmmmmmmmmmmmmmmmmmmmmmmmmmmm",
+    "akOOo",
+    (SUBR) vbap_init,          (SUBR) NULL,    (SUBR) vbap                   },
+  { "vbap.A",      S(VBAPA), TR, 5,  "a[]",    "akOOo",
+    (SUBR) vbap_init_a,          (SUBR) NULL,    (SUBR) vbap_a               },
+  { "vbap4",      S(VBAP),
+    TR|_QQ, 5,  "aaaa",  "akOOo", (SUBR) vbap_init, (SUBR) NULL, (SUBR) vbap },
+  { "vbap8",      S(VBAP),
+    TR|_QQ, 5,  "aaaaaaaa", "akOOo",
+    (SUBR) vbap_init,          (SUBR) NULL,    (SUBR) vbap                   },
+  { "vbap16",      S(VBAP),
+    TR|_QQ, 5,  "aaaaaaaaaaaaaaaa", "akOOo",
+    (SUBR) vbap_init,          (SUBR) NULL,    (SUBR) vbap                   },
+  { "vbapg.a",      S(VBAP1),             TR, 3,
+    "zzzzzzzzzzzzzzzzzzzzzzzzzzzzzzzzzzzzzzzzzzzzzzzzzzzzzzzzzzzzzzzz"
+    "zzzzzzzzzzzzzzzzzzzzzzzzzzzzzzzzzzzzzzzzzzzzzzzzzzzzzzzzzzzzzzzz", "kOOo",
+    (SUBR) vbap1_init,         (SUBR) vbap1                                  },
+  { "vbapg.A",      S(VBAPA1),            TR, 3,
+    "k[]",  "kOOo",
+    (SUBR) vbap1_init_a,         (SUBR) vbap1a                               },
+  { "vbapz",      S(VBAP_ZAK),     ZW|TR, 5,  "",                 "iiakOOo",
     (SUBR) vbap_zak_init,           (SUBR) NULL,    (SUBR) vbap_zak         },
-  { "vbaplsinit", S(VBAP_LS_INIT), 1, "", "iioooooooooooooooooooooooooooooooo",
-    (SUBR) vbap_ls_init,            (SUBR) NULL,    (SUBR) NULL             },
-  { "vbap4move",  S(VBAP_FOUR_MOVING),      5,  "aaaa",             "aiiim",
-    (SUBR) vbap_FOUR_moving_init,   (SUBR) NULL,    (SUBR) vbap_FOUR_moving },
-  { "vbap8move",  S(VBAP_EIGHT_MOVING),     5,  "aaaaaaaa",         "aiiim",
-    (SUBR) vbap_EIGHT_moving_init,  (SUBR) NULL,    (SUBR) vbap_EIGHT_moving },
-  { "vbap16move", S(VBAP_SIXTEEN_MOVING),   5,  "aaaaaaaaaaaaaaaa", "aiiim",
-    (SUBR) vbap_SIXTEEN_moving_init, (SUBR) NULL, (SUBR) vbap_SIXTEEN_moving },
-  { "vbapzmove",  S(VBAP_ZAK_MOVING),       5,  "",                 "iiaiiim",
-    (SUBR) vbap_zak_moving_init,    (SUBR) NULL,    (SUBR) vbap_zak_moving  }
+  { "vbaplsinit",S(VBAP_LS_INIT),TR,1, "",
+    "ii"
+    "oooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooo"
+    "oooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooo",
+    (SUBR) vbap_ls_init, (SUBR) NULL, (SUBR) NULL, (SUBR) NULL         },
+  { "vbaplsinit",S(VBAP_LS_INIT),TR,1, "", "iii[]",
+    (SUBR) vbap_ls_inita, (SUBR) NULL, (SUBR) NULL, (SUBR) NULL         },
+  { "vbapmove.a", S(VBAP_MOVING),
+    TR, 5,  "mmmmmmmmmmmmmmmmmmmmmmmmmmmmmmmmmmmmmmmmmmmmmmmmmmmmmmmmmmmmmmmm"
+    "mmmmmmmmmmmmmmmmmmmmmmmmmmmmmmmmmmmmmmmmmmmmmmmmmmmmmmmmmmmmmmmm",
+    "aiiim",
+    (SUBR) vbap_moving_init, (SUBR) NULL, (SUBR) vbap_moving },
+  { "vbapgmove.a",  S(VBAP1_MOVING),      TR, 5,
+    "zzzzzzzzzzzzzzzzzzzzzzzzzzzzzzzzzzzzzzzzzzzzzzzzzzzzzzzzzzzzzzzz"
+    "zzzzzzzzzzzzzzzzzzzzzzzzzzzzzzzzzzzzzzzzzzzzzzzzzzzzzzzzzzzzzzzz", "iiim",
+    (SUBR) vbap1_moving_init,   (SUBR) NULL,    (SUBR) vbap1_moving },
+  { "vbapmove.A", S(VBAPA_MOVING),
+    TR, 5,  "a[]",  "aiiim",
+    (SUBR) vbap_moving_init_a, (SUBR) NULL, (SUBR) vbap_moving_a },
+  { "vbapgmove.A",  S(VBAPA1_MOVING),      TR, 5,
+    "k[]", "iiim",
+    (SUBR) vbap1_moving_init_a,   (SUBR) NULL,    (SUBR) vbap1_moving_a },
+  { "vbapzmove",  S(VBAP_ZAK_MOVING),    ZW|TR, 5,  "",  "iiaiiim",
+    (SUBR) vbap_zak_moving_init,    (SUBR) NULL,    (SUBR) vbap_zak_moving  },
+  { "vbap4move", S(VBAP_MOVING),   TR|_QQ, 5,  "aaaa",
+   "aiiim",
+    (SUBR) vbap_moving_init, (SUBR) NULL, (SUBR) vbap_moving },
+  { "vbap8move", S(VBAP_MOVING),
+    TR|_QQ, 5,  "aaaaaaaa",
+    "aiiim",
+    (SUBR) vbap_moving_init, (SUBR) NULL, (SUBR) vbap_moving }
+
 };
 
-PUBLIC long csound_opcode_init(CSOUND *csound, OENTRY **ep)
-{
-    create_ls_table(csound, 3);
-    *ep = localops;
-    return (long) sizeof(localops);
-}
-
-PUBLIC int csoundModuleInfo(void)
-{
-    return ((CS_APIVERSION << 16) + (CS_APISUBVER << 8) + (int) sizeof(MYFLT));
-}
+LINKAGE_BUILTIN(vbap_localops)
 
