@@ -259,15 +259,15 @@ static CS_NOINLINE int csoundLoadExternal(CSOUND *csound,
                libraryPath, err);
  #endif
       if (csound->delayederrormessages == NULL) {
-        csound->delayederrormessages = malloc(strlen(ERRSTR)+1);
+        csound->delayederrormessages = csound->Malloc(csound, strlen(ERRSTR)+1);
         strcpy(csound->delayederrormessages, ERRSTR);
       }
       else {
         char *new =
-          realloc(csound->delayederrormessages,
-                  strlen(csound->delayederrormessages)+strlen(ERRSTR)+11);
+          csound->ReAlloc(csound, csound->delayederrormessages,
+                          strlen(csound->delayederrormessages)+strlen(ERRSTR)+11);
         if (new==NULL) {
-          free(csound->delayederrormessages);
+          csound->Free(csound, csound->delayederrormessages);
           return CSOUND_ERROR;
         }
         csound->delayederrormessages = new;
@@ -323,7 +323,8 @@ static CS_NOINLINE int csoundLoadExternal(CSOUND *csound,
     }
     /* set up module info structure */
     /* (note: space for NUL character is already included in size of struct) */
-    p = (void*) malloc(sizeof(csoundModule_t) + (size_t) strlen(fname));
+    p = (void*) csound->Malloc(csound,
+                               sizeof(csoundModule_t) + (size_t) strlen(fname));
     if (UNLIKELY(p == NULL)) {
       csoundCloseLibrary(h);
       csound->ErrorMsg(csound,
@@ -358,7 +359,7 @@ static CS_NOINLINE int csoundLoadExternal(CSOUND *csound,
     return CSOUND_SUCCESS;
 }
 
-static int csoundCheckOpcodeDeny(const char *fname)
+static int csoundCheckOpcodeDeny(CSOUND * csound, const char *fname)
 {
     /* Check to see if the fname is on the do-not-load list */
     char buff[256];
@@ -370,19 +371,19 @@ static int csoundCheckOpcodeDeny(const char *fname)
     if (list==NULL) return 0;
     strncpy(buff, fname, 255); buff[255]='\0';
     strrchr(buff, '.')[0] = '\0'; /* Remove .so etc */
-    p = strdup(list);
+    p = cs_strdup(csound, list);
     deny = cs_strtok_r(p, ",", &th);
     /* printf("DEBUG %s(%d): check buff=%s\n", __FILE__, __LINE__, deny); */
     while (deny) {
       /* printf("DEBUG %s(%d): deny=%s\n", __FILE__, __LINE__, deny); */
       if (strcmp(deny, buff)==0) {
-        free(p);
+        csound->Free(csound, p);
         /* printf("DEBUG %s(%d): found\n", __FILE__, __LINE__); */
         return 1;
       }
       deny = cs_strtok_r(NULL, ",", &th);
     }
-    free(p);
+    csound->Free(csound, p);
     /* printf("DEBUG %s(%d): not found\n", __FILE__, __LINE__); */
     return 0;
 }
@@ -470,7 +471,7 @@ int csoundLoadModules(CSOUND *csound)
         continue;
       }
       /* printf("DEBUG %s(%d): possibly deny %s\n", __FILE__, __LINE__,fname); */
-      if (csoundCheckOpcodeDeny(fname)) {
+      if (csoundCheckOpcodeDeny(csound, fname)) {
         csoundWarning(csound, Str("Library %s omitted\n"), fname);
         continue;
       }
@@ -513,7 +514,7 @@ int csoundLoadExternals(CSOUND *csound)
       if (s[i] == ',')
         cnt++;
     } while (s[++i] != '\0');
-    lst = (char**) malloc(sizeof(char*) * cnt);
+    lst = (char**) csound->Malloc(csound, sizeof(char*) * cnt);
     i = cnt = 0;
     lst[cnt++] = s;
     do {
@@ -535,7 +536,7 @@ int csoundLoadExternals(CSOUND *csound)
       }
     } while (++i < cnt);
     /* file list is no longer needed */
-    free(lst);
+    csound->Free(csound, lst);
     csound->Free(csound, s);
     return 0;
 }
@@ -667,7 +668,7 @@ int csoundDestroyModules(CSOUND *csound)
       csoundCloseLibrary(m->h);
       csound->csmodule_db = (void*) m->nxt;
       /* free memory used by database */
-      free((void*) m);
+      csound->Free(csound, (void*) m);
 
     }
     sfont_ModuleDestroy(csound);
@@ -798,7 +799,7 @@ extern long hrtfreverb_localops_init(CSOUND *, void *);
 extern long hrtfearly_localops_init(CSOUND *, void *);
 extern long minmax_localops_init(CSOUND *, void *);
 
-extern long stackops_localops_init(CSOUND *, void *);
+//extern long stackops_localops_init(CSOUND *, void *);
 extern long vbap_localops_init(CSOUND *, void *);
 extern long vaops_localops_init(CSOUND *, void*);
 extern long ugakbari_localops_init(CSOUND *, void *);
@@ -822,6 +823,7 @@ extern long sockrecv_localops_init(CSOUND *, void *);
 extern long afilts_localops_init(CSOUND *, void *);
 extern long pinker_localops_init(CSOUND *, void *);
 extern long paulstretch_localops_init(CSOUND *, void *);
+extern long wpfilters_localops_init(CSOUND *, void *);
 
 extern int stdopc_ModuleInit(CSOUND *csound);
 extern int pvsopc_ModuleInit(CSOUND *csound);
@@ -839,7 +841,7 @@ const INITFN staticmodules[] = { hrtfopcodes_localops_init, babo_localops_init,
                                  hrtferX_localops_init, loscilx_localops_init,
                                  pan2_localops_init, arrayvars_localops_init,
                                  phisem_localops_init, pvoc_localops_init,
-                                 stackops_localops_init, vbap_localops_init,
+                                 /*stackops_localops_init,*/ vbap_localops_init,
                                  ugakbari_localops_init, harmon_localops_init,
                                  pitchtrack_localops_init, partikkel_localops_init,
                                  shape_localops_init, tabsum_localops_init,
@@ -859,6 +861,7 @@ const INITFN staticmodules[] = { hrtfopcodes_localops_init, babo_localops_init,
                                  gendy_localops_init,
                                  scnoise_localops_init, afilts_localops_init,
                                  pinker_localops_init,
+                                 wpfilters_localops_init,
                                  NULL };
 
 typedef NGFENS* (*FGINITFN)(CSOUND *);
