@@ -116,8 +116,8 @@ static  int     pvxanal(CSOUND *csound, SOUNDIN *p, SNDFILE *fd,
                                         const char *fname,
                                         long srate, long chans, long fftsize,
                                         long overlap, long winsize,
-                                        pv_wtype wintype, int verbose,
-                                        double beta);
+                                        pv_wtype wintype,
+                                        double beta, int displays);
 static  long    generate_frame(CSOUND*, PVX *pvx, MYFLT *fbuf, float *outanal,
                                         long samps, int frametype);
 static  void    chan_split(CSOUND*, const MYFLT *inbuf, MYFLT **chbuf,
@@ -162,10 +162,10 @@ static int pvanal(CSOUND *csound, int argc, char **argv)
     FILE    *trfil = stdout;
     pv_wtype  WindowType = PVOC_HANN;
     char    err_msg[512];
-    int     verbose = 0;
     double  beta = 6.8;
+    int displays = 0;
+
 
-    csound->oparms->displays = 0;
     if (!(--argc))
       return quit(csound, Str("insufficient arguments"));
       do {
@@ -174,9 +174,9 @@ static int pvanal(CSOUND *csound, int argc, char **argv)
           switch (*s++) {
           case 's': FIND(Str("no sampling rate"));
 #if defined(USE_DOUBLE)
-            sscanf(s, "%lf", &sr);
+            csound->sscanf(s, "%lf", &sr);
 #else
-            sscanf(s, "%f", &sr);
+            csound->sscanf(s, "%f", &sr);
 #endif
             break;
           case 'c':  FIND(Str("no channel"));
@@ -184,16 +184,16 @@ static int pvanal(CSOUND *csound, int argc, char **argv)
             break;
           case 'b':  FIND(Str("no begin time"));
 #if defined(USE_DOUBLE)
-            sscanf(s, "%lf", &beg_time);
+            csound->sscanf(s, "%lf", &beg_time);
 #else
-            sscanf(s, "%f", &beg_time);
+            csound->sscanf(s, "%f", &beg_time);
 #endif
             break;
           case 'd':  FIND(Str("no duration time"));
 #if defined(USE_DOUBLE)
-            sscanf(s, "%lf", &input_dur);
+            csound->sscanf(s, "%lf", &input_dur);
 #else
-            sscanf(s, "%f", &input_dur);
+            csound->sscanf(s, "%f", &input_dur);
 #endif
             break;
           case 'H':
@@ -204,12 +204,12 @@ static int pvanal(CSOUND *csound, int argc, char **argv)
             break;
           case 'B':
             FIND(Str("no beta given"));
-            sscanf(s, "%lf", &beta);
+            csound->sscanf(s, "%lf", &beta);
             break;
           case 'n':  FIND(Str("no framesize"));
             sscanf(s, "%ld", &frameSize);
             if (frameSize < MINFRMPTS || frameSize > MAXFRMPTS) {
-              sprintf(err_msg, Str("frameSize must be between %d and %d"),
+              snprintf(err_msg, 512, Str("frameSize must be between %d and %d"),
                                MINFRMPTS, MAXFRMPTS);
               return quit(csound, err_msg);
             }
@@ -222,11 +222,11 @@ static int pvanal(CSOUND *csound, int argc, char **argv)
           case 'h':  FIND(Str("no hopsize"));
             sscanf(s, "%ld", &frameIncr);
             break;
-          case 'g':  csound->oparms->displays = 1;
+          case 'g':  displays = 1;
             break;
           case 'G':  FIND(Str("no latch"));
             sscanf(s, "%d", &latch);
-            csound->oparms->displays = 1;
+            displays = 1;
             break;
           case 'V':  FIND(Str("no output file for trace"));
             {
@@ -236,9 +236,6 @@ static int pvanal(CSOUND *csound, int argc, char **argv)
                 return quit(csound, Str("Failed to open text file"));
               csound->Message(csound, Str("Writing text form to file %s\n"), s);
             }
-          case 'v':
-            verbose = 1;
-            break;
           default:
             return quit(csound, Str("unrecognised switch option"));
           }
@@ -255,7 +252,7 @@ static int pvanal(CSOUND *csound, int argc, char **argv)
     /* open sndfil, do skiptime */
     if ((infd = csound->SAsndgetset(csound, infilnam, &p, &beg_time,
                                     &input_dur, &sr, channel)) == NULL) {
-      sprintf(err_msg, Str("error while opening %s"), infilnam);
+      snprintf(err_msg, 512, Str("error while opening %s"), infilnam);
       return quit(csound, err_msg);
     }
     sr = (MYFLT)p->sr;
@@ -268,6 +265,7 @@ static int pvanal(CSOUND *csound, int argc, char **argv)
         frameSize >>= 1;        /* divide down until just larger */
     }
     if (ovlp == 0 && frameIncr == 0) {
+      csound->Message(csound, "frameSize=%ld\n", frameSize);
       ovlp = OVLP_DEF;          /* default overlap */
       frameIncr = frameSize / ovlp;
     }
@@ -276,9 +274,12 @@ static int pvanal(CSOUND *csound, int argc, char **argv)
     else frameIncr = frameSize/ovlp;
 
     if (ovlp < 2 || ovlp > 64) {
-      csound->Message(csound, Str("pvanal: %d is a bad window overlap index\n"),
-                              (int) ovlp);
-      return -1;
+      csound->Message(csound,
+                      Str("WARNING: pvanal: %d might be a bad window "
+                          "overlap index\n"),
+                      (int) ovlp);
+      /* return -1; */
+      /* VL: removed this restriction, which sounds a bit drastic */
     }
     oframeEst = (p->getframes - frameSize/2) / frameIncr;
     csound->Message(csound, Str("%ld infrsize, %ld infrInc\n"),
@@ -294,34 +295,34 @@ static int pvanal(CSOUND *csound, int argc, char **argv)
     }
     csound->Message(csound, Str("pvanal: creating pvocex file\n"));
     /* handle all messages in here, for now */
-    if (csound->oparms->displays)
-      csound->dispinit(csound);
+    if (displays)
+        csound->dispinit(csound);
     if (pvxanal(csound, p, infd, outfilnam, p->sr,
                         ((!channel || channel == ALLCHNLS) ? p->nchanls : 1),
                         frameSize, frameIncr, frameSize * 2,
-                        WindowType, verbose, beta) != 0) {
+                WindowType, beta, displays) != 0) {
       csound->Message(csound, Str("error generating pvocex file.\n"));
       return -1;
     }
-    if (csound->oparms->displays)
+    if (displays)
       csound->dispexit(csound);
 
     return 0;
 }
 
 static const char *pvanal_usage_txt[] = {
-    "Usage: pvanal [options...] inputSoundfile outputFFTfile.pvx",
-    "Options:",
-    "    -c <channel>",
-    "    -b <beginTime>",
-    "    -d <duration>",
-    "    -n <frameSize>",
-    "    -w <windowOverlap> | -h <hopSize>",
-    "    -g | -G <latch>",
-    "    -v | -V <txtFile>",
-    "    -H: use Hamming window instead of the default (von Hann)",
-    "    -K: use Kaiser window",
-    "    -B <beta>: parameter for Kaiser window",
+  Str_noop("Usage: pvanal [options...] inputSoundfile outputFFTfile.pvx"),
+  Str_noop("Options:"),
+  Str_noop("    -c <channel>"),
+  Str_noop("    -b <beginTime>"),
+  Str_noop("    -d <duration>"),
+  Str_noop("    -n <frameSize>"),
+  Str_noop("    -w <windowOverlap> | -h <hopSize>"),
+  Str_noop("    -g | -G <latch>"),
+  Str_noop("    -v | -V <txtFile>"),
+  Str_noop("    -H: use Hamming window instead of the default (von Hann)"),
+  Str_noop("    -K: use Kaiser window"),
+  Str_noop("    -B <beta>: parameter for Kaiser window"),
     NULL
 };
 
@@ -329,7 +330,7 @@ static int quit(CSOUND *csound, char *msg)
 {
     int i;
 
-    csound->Message(csound, "pvanal error: %s\n", msg);
+    csound->Message(csound, Str("pvanal error: %s\n"), msg);
     for (i = 0; pvanal_usage_txt[i] != NULL; i++)
       csound->Message(csound, "%s\n", Str(pvanal_usage_txt[i]));
     return -1;
@@ -342,7 +343,7 @@ int pvanal_init_(CSOUND *csound)
     int retval = csound->AddUtility(csound, "pvanal", pvanal);
     if (!retval) {
       retval = csound->SetUtilityDescription(csound, "pvanal",
-                                             "Soundfile analysis for pvoc");
+                                             Str("Soundfile analysis for pvoc"));
     }
     return retval;
 }
@@ -361,8 +362,6 @@ static void PVDisplay_Init(CSOUND *csound, PVDISPLAY *p,
 
     memset(p, 0, sizeof(PVDISPLAY));
     p->csound = csound;
-    if (!csound->oparms->displays)
-      return;
     p->npts = (fftSize / 2) + 1;
     p->dispCntMax = dispCntMax;
     for (i = 0; i < DISPFRAMES; i++)
@@ -373,7 +372,7 @@ static void PVDisplay_Update(PVDISPLAY *p, const float *buf)
 {
     int     i;
 
-    if (!p->csound->oparms->displays || p->dispFrame >= DISPFRAMES)
+    if (p->dispFrame >= DISPFRAMES)
       return;
     for (i = 0; i < p->npts; i++)
       p->dispBufs[p->dispFrame][i] += ((MYFLT) buf[i * 2] * (MYFLT) buf[i * 2]);
@@ -384,7 +383,7 @@ static void PVDisplay_Display(PVDISPLAY *p, int frame)
 {
     int     i;
 
-    if (!p->csound->oparms->displays || p->dispFrame >= DISPFRAMES ||
+    if (p->dispFrame >= DISPFRAMES ||
         p->dispCnt < p->dispCntMax)
       return;
     for (i = 0; i < p->npts; i++)
@@ -393,7 +392,7 @@ static void PVDisplay_Display(PVDISPLAY *p, int frame)
                                  / (MYFLT) p->dispCnt));
     p->csound->dispset(p->csound, &(p->dwindow), p->dispBufs[p->dispFrame],
                        p->npts, "pvanalwin", 0, "PVANAL");
-    sprintf(&(p->dwindow.caption[0]), "%ld", (long) frame);
+    snprintf(&(p->dwindow.caption[0]), CAPSIZE, "%ld", (long) frame);
     p->csound->display(p->csound, &(p->dwindow));
     p->dispCnt = 0;
     p->dispFrame++;
@@ -401,12 +400,11 @@ static void PVDisplay_Display(PVDISPLAY *p, int frame)
 
 /* Only supports PVOC_AMP_FREQ format for now */
 
-/* not sure how to use 'verbose' yet; but it's here...*/
 /* cannot add display code, as we may have 8 channels here...*/
 
 static int pvxanal(CSOUND *csound, SOUNDIN *p, SNDFILE *fd, const char *fname,
                    long srate, long chans, long fftsize, long overlap,
-                   long winsize, pv_wtype wintype, int verbose, double beta)
+                   long winsize, pv_wtype wintype, double beta, int displays)
 {
     int         i, k, pvfile = -1, rc = 0;
     pv_stype    stype = STYPE_16;
@@ -417,10 +415,8 @@ static int pvxanal(CSOUND *csound, SOUNDIN *p, SNDFILE *fd, const char *fname,
     MYFLT       *inbuf_c[MAXPVXCHANS];
     float       *frame_c[MAXPVXCHANS];  /* RWD : MUST be 32bit  */
     MYFLT       *inbuf = NULL;
-    MYFLT       nyquist, chwidth;
     float       *frame;                 /* RWD : MUST be 32bit  */
     MYFLT       *chanbuf;
-    unsigned int  nbins;
     long        total_sampsread = 0;
     PVDISPLAY   disp;
 
@@ -431,9 +427,9 @@ static int pvxanal(CSOUND *csound, SOUNDIN *p, SNDFILE *fd, const char *fname,
       case AE_FLOAT:  stype = STYPE_IEEE_FLOAT; break;
     }
 
-    nbins = (fftsize/2) + 1;
-    nyquist = (MYFLT) srate * FL(0.5);
-    chwidth = nyquist/(MYFLT)(nbins - 1);
+    //nbins = (fftsize/2) + 1;
+    //nyquist = (MYFLT) srate * FL(0.5);
+    //    chwidth = nyquist/(MYFLT)(nbins - 1);
     for (i = 0; i < MAXPVXCHANS; i++) {
       pvx[i] = NULL;
       inbuf_c[i] = NULL;
@@ -471,7 +467,7 @@ static int pvxanal(CSOUND *csound, SOUNDIN *p, SNDFILE *fd, const char *fname,
       rc = 1;
       goto error;
     }
-
+    if (displays)
     PVDisplay_Init(csound, &disp, (int) fftsize,
                    (int) (((long) p->getframes * chans / overlap)
                           / DISPFRAMES));
@@ -481,8 +477,9 @@ static int pvxanal(CSOUND *csound, SOUNDIN *p, SNDFILE *fd, const char *fname,
       total_sampsread += sampsread;
       /* zeropad to full buflen */
       if (sampsread < buflen_samps) {
-        for (i = sampsread; i < buflen_samps; i++)
-          inbuf[i] = FL(0.0);
+        /* for (i = sampsread; i < buflen_samps; i++) */
+        /*   inbuf[i] = FL(0.0); */
+        memset(inbuf, 0, sizeof(MYFLT)*buflen_samps);
         sampsread = buflen_samps;
       }
       chan_split(csound, inbuf, inbuf_c, sampsread, chans);
@@ -502,12 +499,12 @@ static int pvxanal(CSOUND *csound, SOUNDIN *p, SNDFILE *fd, const char *fname,
             goto error;
           }
           blocks_written++;
-          PVDisplay_Update(&disp, frame);
+          if (displays) PVDisplay_Update(&disp, frame);
           if ((blocks_written/chans) % 20 == 0) {
             csound->Message(csound, "%ld\n", blocks_written/chans);
+          }
+          if (displays) PVDisplay_Display(&disp, (int) (blocks_written / chans));
         }
-        PVDisplay_Display(&disp, (int) (blocks_written / chans));
-      }
       }
       if (total_sampsread >= p->getframes*chans)
         break;
@@ -515,8 +512,9 @@ static int pvxanal(CSOUND *csound, SOUNDIN *p, SNDFILE *fd, const char *fname,
 
     /* write out remaining frames */
     sampsread = fftsize * chans;
-    for (i = 0;i< sampsread;i++)
-      inbuf[i] = FL(0.0);
+    /* for (i = 0;i< sampsread;i++) */
+    /*   inbuf[i] = FL(0.0); */
+    memset(inbuf, 0, sizeof(MYFLT)*sampsread);
     chan_split(csound,inbuf,inbuf_c,sampsread,chans);
     for (i = 0; i < sampsread/chans; i+= overlap) {
       for (k = 0; k < chans; k++) {
@@ -533,9 +531,9 @@ static int pvxanal(CSOUND *csound, SOUNDIN *p, SNDFILE *fd, const char *fname,
           goto error;
         }
         blocks_written++;
-        PVDisplay_Update(&disp, frame);
+        if (displays) PVDisplay_Update(&disp, frame);
       }
-      PVDisplay_Display(&disp, (int) (blocks_written / chans));
+      if (displays) PVDisplay_Display(&disp, (int) (blocks_written / chans));
     }
     csound->Message(csound, Str("\n%ld %d-chan blocks written to %s\n"),
                     (long) blocks_written / (long) chans, (int) chans, fname);
@@ -688,15 +686,15 @@ static int init(CSOUND *csound,
     thispvx->Dd = thispvx->analWinLen + thispvx->nI + 1;
     thispvx->flag = 1;
 
-    for (i=0;i < thispvx->ibuflen;i++)
-      thispvx->input[i] = FL(0.0);
-
-    for (i=0;i < N+2;i++)
-      thispvx->anal[i] = FL(0.0);
-
-    for (i=0;i < N2+1;i++)
-      thispvx->oldInPhase[i] = FL(0.0);
-
+    /* for (i=0;i < thispvx->ibuflen;i++) */
+    /*   thispvx->input[i] = FL(0.0); */
+    memset(thispvx->input, 0, sizeof(MYFLT)*thispvx->ibuflen);
+    /* for (i=0;i < N+2;i++) */
+    /*   thispvx->anal[i] = FL(0.0); */
+    memset(thispvx->anal, 0, sizeof(MYFLT)*(N+2));
+    /* for (i=0;i < N2+1;i++) */
+    /*   thispvx->oldInPhase[i] = FL(0.0); */
+    memset(thispvx->oldInPhase, 0, sizeof(MYFLT)*(N2+1));
     *pvx = thispvx;
     return 0;
 }
@@ -756,8 +754,10 @@ static long generate_frame(CSOUND *csound, PVX *pvx,
        channels.   The subroutines fft and reals together implement
        one efficient FFT call for a real input sequence.  */
 
-    for (i = 0; i < N+2; i++)
-      *(anal + i) = FL(0.0);        /*initialize*/
+/* initialize */
+    /* for (i = 0; i < N+2; i++) */
+    /*   *(anal + i) = FL(0.0); */
+    memset(anal, 0, sizeof(MYFLT)*(N+2));
 
     j = (pvx->nI - pvx->analWinLen-1+pvx->ibuflen)%pvx->ibuflen;  /*input pntr*/
 
@@ -823,18 +823,19 @@ static void chan_split(CSOUND *csound, const MYFLT *inbuf, MYFLT **chbuf,
 {
     long i,j,len;
     MYFLT ampfac;
-    MYFLT *buf_c[MAXPVXCHANS];
+    MYFLT *buf_c[MAXPVXCHANS] = { NULL };
+
     const MYFLT *p_inbuf = inbuf;
 
     len = insize/chans;
-    ampfac = csound->dbfs_to_float;
+    ampfac = (1.0/csound->Get0dBFS(csound));
 
     for (i=0;i < chans;i++)
       buf_c[i] = chbuf[i];
 
     for (i = len;i;--i)
       for (j=0;j < chans;j++)
-        *(buf_c[j]++) = ampfac * *p_inbuf++;
+        *(buf_c[j]++) = ampfac * *p_inbuf++; /* ** this line is odd ** */
 
 }
 
@@ -921,4 +922,3 @@ static void vonhann(MYFLT *win, int winLen, int even)
         win[i] = (MYFLT)(0.5 + 0.5 * cos(ftmp*(double)i));
     }
 }
-
