@@ -27,6 +27,11 @@
 #include "scot.h"
 
 #define Str(x)  (x)
+#if defined(HAVE_GCC3)
+#  define UNLIKELY(x)   __builtin_expect(!!(x),0)
+#else
+#  define UNLIKELY(x)   x
+#endif
 
 static  char    curline[MAXLINE + 1];   /* current line of infile */
 static  int     inx,                    /* column # */
@@ -677,7 +682,7 @@ void addparam(int n,            /* number of param to change */
 
     ps = findparam(n, ptop);
     if (strcmp(s, "."))
-      strcpy(ps, s);
+      strncpy(ps, s, 1+strlen(s));
 }
 
 static                          /* returns pointer to */
@@ -739,6 +744,7 @@ int applymacs(char **s, Inst *n)
     for (sz = 0; (*s)[sz]; sz++) {
       if (sz >= 300) {
         scoterror(Str("Macro expansion too long -- circular macros?"));
+        free(news);
         return FALSE;
       }
       news[sz] = (*s)[sz];
@@ -1398,8 +1404,10 @@ void initf(FILE *inf, FILE *outf, char *fil)
     infile = inf;
     outfile = outf;
     infilename = fil;
-    fgets(curline, MAXLINE, infile);
-
+    if (UNLIKELY(NULL == fgets(curline, MAXLINE, infile))) {
+      fprintf(stderr, "Read failure\n");
+      exit(1);
+    }
 #ifdef DEBUG
     printf("1: %s", curline);
 #endif
